/**************************************************************************/
/* FILE NAME: tpu_pwm_example.c               COPYRIGHT (c) MOTOROLA 2002 */
/* VERSION: 1.0                                   All Rights Reserved     */
/*                                                                        */
/* DESCRIPTION: This sample program shows a simple example of a program   */
/* that initializes the PWM function                                      */
/* The example uses the TPU A channel 0 and 1                             */
/* The program is targeted for the MPC555 but should work on any MPC500   */
/* device with a TPU. For other devices the setup routines will also need */
/* to be changed.                                                         */
/*========================================================================*/
/* HISTORY           ORIGINAL AUTHOR: Jeff Loeliger                       */
/* REV      AUTHOR      DATE       DESCRIPTION OF CHANGE                  */
/* ---   -----------  ---------    ---------------------                  */
/* 1.0   R, Dees      17/Oct/02    Initial version.                       */
/**************************************************************************/
#include "mpc555.h"
#include "tpu_pwm.h"
#include "mpc500_util.h"

/* Initialization parameters  */
UINT32 loopctr = 0 ;		// Loop counter for main loop
UINT32 pitctr = 0;		// Counter for number of PIT timeouts
struct TPU3_tag *tpua = &TPU_A;   /* pointer for TPU routines */

/* Example specific values */
UINT8 chan_a = 0;
UINT8 chan_b = 1;
UINT16 init_period = 0x2000; /* This generates a 1.2 KHz square wave initially. */ 
UINT16 init_hightime = 0x1000;
UINT16 step_size = 0x100;

void init555(int freq)
{
    USIU.SYPCR.R = 0xffffff03;  /* Disable watchdog timer.                 *
                                 * WARNING: this is a WRITE ONLY register. */
    USIU.PLPRCRK.R = 0x55ccaa33;  /*Unlock PLPRC with special key */
if (freq == 56){
    USIU.PLPRCR.R = 0x00d3d000; /* 14x PLL operation on normal power mode */
}else
{
    USIU.PLPRCR.R = 0x0093d000; /* 10x PLL operation on normal power mode */
}

	UIMB.UMCR.B.HSPEED = 0;		// Make IMB clock = System Clock
}
void initPIT(void) 
{
							// STEP 1: MODULE SPECIFIC INITIALIZATION
   USIU.PITC.B.PITC = 15624;	// Setup count value - 1 second. 
   USIU.PISCR.B.PITF = 1;	// Freeze enabled to stop PIT
   USIU.PISCR.B.PTE = 1;	// PIT enabled to start counting	  
   
							// STEP 2: LEVEL ASSIGNMENT
   USIU.PISCR.B.PIRQ = 0x80; // Level 0 PIT interrupt

							// STEP 3: ENABLE INTERRUPT
   USIU.PISCR.B.PIE = 1 ;	// Enable PIT interrupt

							// STEP 4: SET APPROPRIATE SIMASK BITS
   USIU.SIMASK.R = 0x40000000; 	// Enable level 0; others disabled
} 

void pit_handler (void)
{
UINT16 old_hightime, new_hightime, period;

/* Toggle the mios PIO bits 11/12 for activity */	
	MIOS1.MPIOSM32DR.B.D11 = !MIOS1.MPIOSM32DR.B.D11;
/* do second mios pin at end to show the time difference in the ineterupt. */

	pitctr++;				/* Increment PIT counter */
	USIU.PISCR.B.PS = 1;	/* Negate PIT flag by writing "1" to it */

/* get current hightime and period - assumes both channels are the same */	
	old_hightime = tpu_pwm_get_hightime(tpua,0);
	period = tpu_pwm_get_period(tpua,0);
	
	if (old_hightime > (period - step_size))
	{
		new_hightime = step_size;
	}
	else
	{
		new_hightime = old_hightime + step_size;
	}
/* do first channel with normal update */
	tpu_pwm_update(tpua,chan_a,period, new_hightime, TPU_PWM_NORMAL);
/* do second channel with immediate update */
	tpu_pwm_update(tpua,chan_b,period, new_hightime, TPU_PWM_IMMED);

/* Now toggle the other MIOS pin */
	MIOS1.MPIOSM32DR.B.D12 = !MIOS1.MPIOSM32DR.B.D12;
		
}

/*************** MIOS PIO ***************/
/* Set up the MIOS Parallel port */
/* set bits 11 and 12 to Outputs and initialize to 11 high, 12 low */
void Init_MIOS_PIO (void)
{ 

/* set MIOS Parallel Port bits 11 and 12 to be outputs */
    MIOS1.MPIOSM32DDR.B.DDR11 = 1; /* setup MIOS Parallel Port bit 11 output */
    MIOS1.MPIOSM32DDR.B.DDR12 = 1; /* setup MIOS Parallel Port bit 12 output */

/* set initial condition of the GPIO bits (LEDs) */    
    MIOS1.MPIOSM32DR.B.D11 = 1;
    MIOS1.MPIOSM32DR.B.D12 = 0;
}

/************** TPU Set up **************/
/* Set up the TPU....                      */

void setup_tpu(struct TPU3_tag *tpu)
{
     tpu->TPUMCR.R = 0x2020; /* TCR1 prescaler divide by 2, supervisor and user acces. */
     tpu->TPUMCR3.R = 0x0040; /* enable enhanced prescaler - divide by 2 */
     tpu->TICR.B.CIRL = 5; /* set interupt level to 5.... */
     tpu->TICR.B.ILBS = 0; /* (but this example will not use TPU Interupt) */
}


void main()
{
	init555(40);				/* Perform a simple 555 initialization */
	Init_MIOS_PIO();
	initPIT();				/* Init PIT to generate interrupts */
	setup_tpu(tpua); /* Do general TPU set up. */
	tpu_pwm_init(tpua, chan_a, TPU_PRIORITY_HIGH, init_period, init_hightime, TPU_PWM_TCR1);
	tpu_pwm_init(tpua, chan_b, TPU_PRIORITY_HIGH, init_period, init_hightime, TPU_PWM_TCR1);


#ifndef __MWERKS__
  	asm(" mtspr EIE, r3");	/* FINAL STEP: SET MSR[EE], MSR[RI] BITS */
#endif

#ifdef __MWERKS__

asm{
		mtspr EIE, r3
}
#endif
  	
	while(1)
		{
   		loopctr++;				// Increment loopctr for something to do
		}
}	

/*********************************************************************
 *
 * Copyright:
 *	MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
 * 
 *  Motorola assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************/
