
#include <stdio.h>
#include <string.h>

#include <serial.h>
#include "debugmon.h"
#include "misc.h"
#include "download.h"

void debugmonloop()
{
  int retval = 1;
  char command[MAX_CMD_SIZE+1];
  printf("\n");
  printf("----------------------------------------\n");
  printf("Simple debug monitor\n");
  printf("Type HELP for a list of commands\n");
  printf("Type EXIT to return to main menu\n");
  printf("\n");
  do {
    printf("\ndebug> ");
    readline(command, MAX_CMD_SIZE, 1);
    retval = do_command(command);
  } while (retval > 0);

}

int do_command(char *str)
{
  if (str[0] == '\0') {
    //empty command, do nothing at all
    return 1;
  }
  else if (strncasecmp(str, "exit", 2) == 0 || strncasecmp(str, "quit", 2) == 0)
    return 0; //quit monitor
  else if (strncasecmp(str, "help", 1) == 0) {
    command_printhelp();
    return 1;
  }
  else if (strncasecmp(str, "mm", 2) == 0) {
    command_modifymem(str+2);
    return 1;
  }
  else if (strncasecmp(str, "md", 2) == 0) {
    command_dumpmem(str+2);
    return 1;
  }
  else if (strncasecmp(str, "bf ", 3) == 0) {
    command_fillmem(str+3);
    return 1;
  }
  else if (strncasecmp(str, "dl", 2) == 0) {
    command_dl(str+2);
    return 1;
  }
  else if (strncasecmp(str, "go ", 3) == 0) {
    command_execute(str+3);
    return 1;
  }
  else {
    printf("Error: Unknown or ambiguous command\n");
    return 1;
  }
}

void command_printhelp()
{
  printf("Debug Monitor Help\n");
  printf("------------------\n");
  printf("\n");
  printf("mm[b|h|w*] <address>   Modify a byte, half word, or word at <address>\n");
  printf("                              Ex: mmw 3fa000   or mmb 0x3fa000\n");
  printf("md[b|h|w*] <address>   View memory as a byte, half word, or word at <address>\n");
  printf("                              Ex: mdh 0x80000\n");
  printf("bf <start> <end> <val> Fill memory from <start> to <end> with <value>\n");
  printf("                              Ex: bf 800000 810000 FFFF\n");
  printf("dl[ir*|er|ef|el]       Download a hex file into internal RAM, external RAM,\n");
  printf("                         external flash, or external relocation (build for\n");
  printf("                         external RAM but write to external flash)\n");
  printf("go <address>           Execute a program at <address>\n");
  printf("                              Ex: go 3fa000\n");
}

void command_modifymem(char *str)
{
  char memsize;
  int strpos = 2; //default to third char
  char *ptr;
  int result;
  char input[16];
  unsigned long val;

  switch (str[0]) {
    case 'b': 
      memsize = 'b';
      break;
    case 'h':
      memsize = 'h';
      break;
    case 'w':
      memsize = 'w';
      break;
    case ' ':
      memsize = 'w';
      strpos--; //no, it's actually the second char
      break;
    default:
      printf("Error: Invalid block size specification (byte, half-word, word)\n");
      return;
      break; //not reached
  }

  ptr = (char *)readhex(str+strpos, &result);
  if (result <= 0) {
    printf("Error: Invalid address specified\n");
    return;
  }

  printf("Current value at 0x%p: ", ptr);

  switch (memsize) {
    case 'b':
      printf("0x%02x\n", *((unsigned char *)ptr));
      break;
    case 'h':
      printf("0x%04x\n", *((unsigned short *)ptr));
      break;
    default: //case 'w':
      printf("0x%08lx\n", *((unsigned long *)ptr));
      break;
  }

  printf("Enter new value at 0x%p: ", ptr);

  readline(input, sizeof(input), 1);
  val = readhex(input, &result);
  if (result <= 0) {
    printf("Error: Invalid value specified\n");
    return;
  }

  switch (memsize) {
    case 'b':
      *((unsigned char *)ptr) = (unsigned char)val;
      break;
    case 'h':
      *((unsigned short *)ptr) = (unsigned short)val;
      break;
    default: //case 'w':
      *((unsigned long *)ptr) = (unsigned long)val;
      break;
  }

  printf("Value changed.\n");
  
}

void command_dumpmem(char *str)
{
  char memsize;
  int strpos = 2; //default to third char
  char *ptr;
  int result;
  unsigned char *ptrb;
  unsigned short *ptrh;
  unsigned long *ptrw;
  int lcount, ccount;

  switch (str[0]) {
    case 'b': 
      memsize = 'b';
      break;
    case 'h':
      memsize = 'h';
      break;
    case 'w':
      memsize = 'w';
      break;
    case ' ':
      memsize = 'w';
      strpos--; //no, it's actually the second char
      break;
    default:
      printf("Error: Invalid block size specification (byte, half-word, word)\n");
      return;
      break; //not reached
  }

  ptr = (char *)readhex(str+strpos, &result);
  if (result <= 0) {
    printf("Error: Invalid address specified\n");
    return;
  }

  printf("\n");

  switch (memsize) {
    case 'b':
      printf("          0  1  2  3  4  5  6  7  8  9  a  b  c  d  e  f\n");
      break;
    case 'h':
      printf("          0 1  2 3  4 5  6 7  8 9  a b  c d  e f\n");
      break;
    default: //case 'w':
      printf("          0 1 2 3  4 5 6 7  8 9 a b  c d e f\n");
      break;
  }

  for (lcount = 0; lcount < 10; lcount++) {
    printf("%p  ", ptr); //%p is equiv to %08x
    switch (memsize) {
      case 'b':
        ptrb = (unsigned char*) ptr;
        for (ccount = 0; ccount < 16; ccount++) {
          printf("%02x ", *ptrb);
          ptrb++;
        }
        break;
      case 'h':
        ptrh = (unsigned short*) ptr;
        for (ccount = 0; ccount < 8; ccount++) {
          printf("%04x ", *ptrh);
          ptrh++;
        }
        break;
      default: //case 'w':
        ptrw = (unsigned long*) ptr;
        for (ccount = 0; ccount < 4; ccount++) {
          printf("%08lx ", *ptrw);
          ptrw++;
        }
        break;
    }
    ptr += 16; //move up 16 bytes
    printf("\n");
  }
  
}

void command_fillmem(char *str)
{
  int strpos;
  unsigned char *startptr, *endptr;
  int result;
  unsigned long val;

  startptr = (unsigned char*)readhex(str, &result);
  if (result <= 0) {
    printf("Error: Invalid start address specified\n");
    return;
  }

//  printf("result = %d\n", result);

  if (str[result] != ' ') {
    printf("Error: Invalid or no end address specified\n");
    return;
  }

  strpos = result+1;

  endptr = (unsigned char*)readhex(str+strpos, &result);
  if (result <= 0) {
    printf("Error: Invalid end address specified\n");
    return;
  }

  if (str[strpos+result] != ' ') {
    printf("Error: Invalid fill value specified\n");
    return;
  }

  strpos += result+1;

  val = readhex(str+strpos, &result);
  if (result <= 0) {
    printf("Error: Invalid fill value specified\n");
    return;
  }

  printf("Filling region 0x%p through 0x%p with value 0x%lx...", startptr, endptr, val);

  if (val < 0x100) {
    //we're filling with a byte
    while (startptr < endptr) {
      *((unsigned char *)startptr) = (unsigned char)val;
      startptr++;
    }
  }
  else if (val < 0x10000) {
    //we're filling with a half-word
    while (startptr < endptr) {
      *((unsigned short *)startptr) = (unsigned short)val;
      startptr += 2;
    }
  }
  else {
    //we're filling with a word
    while (startptr < endptr) {
      *((unsigned long *)startptr) = (unsigned long)val;
      startptr += 4;
    }
  }

  printf("done.\n");

}

void command_execute(char *str)
{
  unsigned char *ptr;
  int result;
  void (*fn)();

  ptr = (unsigned char*)readhex(str, &result);
  if (result <= 0) {
    printf("Error: Invalid address specified\n");
    return;
  }

  fn = (void (*)())ptr;
  fn();

  //we may very well return if the executed program's main returns
}

void command_dl(char *str)
{
  int dltype;

  if (strncasecmp(str, "ir", 2) == 0 || str[0] == '\0') { //default
    //internal RAM
    dltype = DL_INT_RAM;
  }
  else if (strncasecmp(str, "er", 2) == 0) {
    dltype = DL_EXT_RAM;
  }
  else if (strncasecmp(str, "ef", 2) == 0) {
    dltype = DL_EXT_ROM;
  }
  else if (strncasecmp(str, "el", 2) == 0) {
    dltype = DL_EXT_RELOC;
  }
  else {
    printf("Error: Unrecognized download type requested.\n");
    return;
  }

  if (dl_image(dltype, 1) < 0) {
    //error message already printed
  }
  else {
    //download complete and summary message already printed
  }
}

