
#include <stdio.h>
#include <string.h>

#include "download.h"
#include "srec.h"
#include "targetmem.h"
#include "misc.h"
#include "serial.h"

/*
 * Download an image file as srecords from stdin into the approriate memory
 * 
 * dltype is one of the defined download types in download.h
 * verbose is nonzero if a summary should be printed after the download, 0
 *   otherwise
 * 
 * 0 will be returned on success, -1 on error (misformatted s-record, invalid
 *   memory location, etc.) and the appropriate message will be printed to
 *   stdout
 *
 * Note that this function assumes that the S7 record (the record containing
 *   the entry address) is the last line in the input
 *
 * To conserve memory, the input is only buffered a line at a time and the
 *   data written to memory after each line is read.  Thus, if the download
 *   fails for any reason, the target memory will be partially written.
 */
int dl_image(int dltype, int verbose)
{
  char inputline[MAX_SREC_SIZE]; //no +1 needed b/c readline smart
  char data[MAX_SREC_DATA_SIZE]; //not actually a string

  //info about the current srecord
  int srectype;
  int curraddr;
  int datasize;

  //info about the download as a whole
  int addr_upperbound, addr_lowerbound; //_inclusive_
  int addr_offset; //offset to apply to address given in the srec before
                   //  writing it to memory
  char module_name[25] = ""; //the module name contained in the s19 file
                             //  (strictly speaking, unreliable, but GNU uses
                             //  the filename of the s19 file when it was
                             //  written)
  int entry_addr = 0;
  int totalsize = 0;

  //other
  int ret;
  char tmpchar;

  switch (dltype) {
    case DL_INT_RAM:
//      addr_lowerbound = (int)&internal_ram_start;
//      addr_upperbound = (int)&internal_ram_start+(int)&internal_ram_size-1;
//Rather than being strict, we allow the possibility of writing to other bits
//  of internal non-flash memory (i.e., DPTRAM, etc.)

      addr_lowerbound = (int)&internal_memory_start;
      addr_upperbound = (int)&internal_ram_start+(int)&internal_ram_size-1;
      addr_offset = 0;
      printf("Downloading into internal RAM...\n");
      break;
    case DL_INT_ROM:
      printf("Download error: downloading to internal flash is not supported by this version\n");
      return -1;
      break;
    case DL_EXT_RAM:
      //Though we could with some slight modifications, we don't allow writing
      //  to internal RAM also, since we are likely using that ourselves!
      addr_lowerbound = (int)&external_ram_start;
      addr_upperbound = (int)&external_ram_start+(int)&external_ram_size-1;
      addr_offset = 0;
      printf("Downloading into external RAM...\n");
      break;
    case DL_EXT_ROM:
      addr_lowerbound = (int)&external_rom_start;
      addr_upperbound = (int)&external_rom_start+(int)&external_rom_size-1;
      addr_offset = 0;
      printf("Downloading into external ROM...\n");
      //erase ROM
      printf("\nWarning: this procedure will erase the external flash memory!\n");
      while (1) {
        printf("Are you sure you wish to continue? [y/N]: ");
        tmpchar = tolower(GetSByte());
        printf("%c", tmpchar);
        if (tmpchar == 'y') {
          //yes
          printf("\nOk, continuing...\n");
          break;
        }
        else if (tmpchar == 'n' || tmpchar == '\n' || tmpchar == '\r') {
          //no
          printf("\nOk, canceling download...\n");
          return -1;
        }
        else {
          //huh?
          printf("\nInvalid response received.\n");
          //continue
        }
      }
      printf("Erasing external ROM...");
      ret = erase_external_rom();
      if (ret < 0) {
        printf("failed.\n");
        printf("Error erasing external ROM, canceling download.\n");
        return -1;
      }
      else {
        printf("done.\n");
      }
      break;
    case DL_EXT_RELOC:
      addr_lowerbound = (int)&external_ram_start;
      addr_upperbound = (int)&external_ram_start+(int)&external_rom_size-1; //yes, _rom_!
      addr_offset = -(int)&external_ram_start + (int)&external_rom_start;
      printf("Downloading into external ROM for relocation...\n");
      //erase ROM
      printf("\nWarning: this procedure will erase the external flash memory!\n");
      while (1) {
        printf("Are you sure you wish to continue? [y/N]: ");
        tmpchar = tolower(GetSByte());
        printf("%c", tmpchar);
        if (tmpchar == 'y') {
          //yes
          printf("\nOk, continuing...\n");
          break;
        }
        else if (tmpchar == 'n' || tmpchar == '\n' || tmpchar == '\r') {
          //no
          printf("\nOk, canceling download...\n");
          return -1;
        }
        else {
          //huh?
          printf("\nInvalid response received.\n");
          //continue
        }
      }
      printf("Erasing external ROM...");
      ret = erase_external_rom();
      if (ret < 0) {
        printf("failed.\n");
        printf("Error erasing external ROM, canceling download.\n");
        return -1;
      }
      else {
        printf("done.\n");
      }
      break;
    default:
      printf("Download error: unknown download type supplied\n");
      return -1;
      break;
  }

  printf("\n\nSend Motorola hex (S19) file now...\n");

  while (1) {
    readline(inputline, sizeof(inputline), 0);
    //no need to strip CRs or LFs
    if (inputline[0] == '\0') {
      //blank line in the file, ignore it
      continue;
    }
    srectype = srec_parse(inputline, &curraddr, &datasize, data);
    if (srectype < 0) {
      printf("\nError parsing srecord: \"%s\"\n", inputline);
      printf("(mis-formatted s-record or checksum failure)\n");
      return -1;
    }
    else if (srectype == 0) {
      ret = srec_name(inputline, module_name, sizeof(module_name));
      if (ret < 0) {
        //don't consider an error to be fatal; this is just a convenience
        strcpy(module_name, "(unknown)");
      }
    }
    else if (srectype == 7 || srectype == 8 || srectype == 9) {
      entry_addr = curraddr;

      //we assume this terminates the s-records
      break;
    }
    else {
      //this s-record contained actual data

      //check the address for sanity
      if (curraddr < addr_lowerbound || curraddr > addr_upperbound) {
        printf("\nError loading srecord: \"%s\"\n", inputline);
        printf("(the supplied address 0x%08x lies outside the allowed range %08x through %08x)\n", curraddr, addr_lowerbound, addr_upperbound);
        return -1;
      }

      //write the data to memory
      if (dltype == DL_EXT_ROM || dltype == DL_EXT_RELOC) {
        //need to use the external flash programming function

        //enforce the write function's size requirement (integral number of
        //  words)
        if (datasize != (datasize & ~0x3)) {
          printf("\nError loading srecord: \"%s\"\n", inputline);
          printf("(this line does not contain an integral number of words)\n");
          return -1;
        }

        write_external_rom((unsigned int*)(curraddr+addr_offset), (unsigned int*)data, datasize/4);
      }
      else {
        //can do this the easy way
        memcpy((char*)(curraddr+addr_offset), data, datasize);
      }

      totalsize += datasize;
    }

    //tell them we're ready for another
    printf(".");
  }

  //success!

  printf("\n");
  printf("Download successful.\n");
  if (dltype == DL_EXT_ROM || dltype == DL_EXT_RELOC) {
    printf("Resetting external ROM...");
    if (reset_external_rom() < 0) {
      //failed
      printf("failed");
    }
    else {
      //success
      printf("done");
    }
    printf(".\n");
  }

  if (verbose) {
    printf("Summary\n");
    printf("  Bytes written: %d\n", totalsize);
    printf("  Module name  : %s\n", module_name);
    printf("  Entry point  : 0x%x\n", entry_addr);
    printf("  Load point   : 0x%x\n", entry_addr+addr_offset);
  }

  printf("\n");

  return 0;
}

