
#include <stdio.h>
#include <serial.h>
#include "misc.h"

/* 
 * Reads a line from stdin (reads a byte repeatedly until a CR or LF seen)
 *
 * Up to maxlen-1 bytes read from stdin are placed in arr, which will always
 *   be null-terminated (if maxlen == 0, this function has no effect)
 *
 * If echo is nonzero, the characters typed by the user will be echoed back
 *   (with some attempt at intelligent backspace'ing)
 *
 * No trailing CRs or LFs will be included in the returned string
 */
void readline(char *arr, int maxlen, int echo)
{                                   
  char c;
  int i = 0;

  if (maxlen == 0)
    return; //the rest of the code requires that at least one byte be accepted 
 
  while (1) {
    c = GetSByte();
    if (echo) {
      if (c == '\r' || c == '\n') {
        //always echo a newline
        printf("%c", c);
      }
      else if (c == '\b') {
        //backspace hit; clear last read char and restart loop
        if (i > 0) {
          printf("%c", c);
          i--;
        }
        continue;
      }
      else if (c == 0x7F) {
        //delete hit; treat it like a backspace (in case)
        if (i > 0) {
          printf("%c", c);
          i--;
        }
        continue;
      }
      else {
        //echo only if we still have room in the buffer
        if (i < maxlen-1)
          printf("%c", c);
      }
    }
    if (c == '\r' || c == '\n')
      break;
    if (i < maxlen-1)
      arr[i++] = c;
  }

  arr[i] = '\0';
} 

int tohex(char num)
{
  if (num >= '0' && num <= '9')
    return(num - '0');
  else if (num >= 'A' && num <= 'F')
    return(num - 'A' + 10);
  else if (num >= 'a' && num <= 'f')
    return(num - 'a' + 10);
  else
    return -1; //invalid hex character supplied
}

//result is set to the number of characters of the string successfully read
//  (which will be the number of nibbles successfully read, unless there is a
//  prefix 0x, in which case it will be the number of nibbles +2),
//  or , -1 on error (invalid character)
//Up to 8 nibbles can be read.  Any beyond the first 8 will be ignored
unsigned long readhex(char *str, int *result)
{
  int c;
  int i;
  unsigned long val = 0;
  int res = 1;

  if (str[0] == '\0' || str[0] == ' ' || str[0] == '\n' || str[0] == '\r') {
    *result = 0;
    return 0;
  }

  if (str[1] != '\0' && str[0] == '0' && (str[1] == 'x' || str[1] == 'X')) {
    if (str[2] == '\0' || str[2] == ' ' || str[2] == '\n' || str[2] == '\r') {
      *result = -1;
      return 0;
    }
    str += 2;
    res += 2; //offset by two chars
  }

  if ((c = tohex(str[0])) < 0) {
    *result = -1;
    return 0;
  }
  //else,
  val = c;

  for (i = 1; i < 8; i++) {
    if (str[i] == '\0' || str[i] == ' ' || str[i] == '\n' || str[i] == '\r') {
      //end of number
      *result = res;
      return val;
    }
    if ((c = tohex(str[i])) < 0) {
      *result = -1;
      return 0;
    }
    val *= 0x10;
    val += c;
    res++;
  }

  *result = res; //either 8 or 10
  return val;
}

//this will have undesired effects for num > 4
int readhexn(char *str, int num)
{
  int i;
  int val = 0;
  int c;

  for (i = 0; i < 2*num; i++) {
    c = tohex(str[i]);
    if (c < 0)
      return -1;
    val *= 0x10;
    val += c;
  }

  return val;
}

int tolower(int c) {
  if (c >= 'A' && c <= 'Z')
    return(c - 'A' + 'a');
  else
    return(c);
}

int readhexpairs(void *dest, char *str, int num)
{
  int i;
  int c1, c2;
  char *destp;

  destp = dest;

  for (i = 0; i < num; i++) {
    c1 = tohex(str[2*i]);
    if (c1 < 0)
      return -1;
    c2 = tohex(str[2*i+1]);
    if (c2 < 0)
      return -1;
    *destp++ = (c1 * 0x10) + c2;
  }

  return 0;
}

int readhexbyte(char *str)
{
  int c1, c2;
    
  c1 = tohex(str[0]);
  if (c1 < 0)
    return -1;
  c2 = tohex(str[1]);
  if (c2 < 0)
    return -1;
  return (c1 * 0x10) + c2;
}

