
#include "srec.h"
#include "misc.h"


/*
 * Takes in a single s-record (one line of an s-record file) as str
 * The intended address of the data as specified in the s-record will be
 *   placed in addrloc
 * The number of bytes of data written to dest will be placed in addrloc
 * The data contained in the s-record will be placed in dest
 *
 * dest should be able to hold up to 255 bytes, though at least one source
 *   states that only 64 bytes are necessary.
 *
 * The numeric type (i.e., 0-9) will be returned on succcess, or -1 on error
 *   (improperly formatted, truncated, checksum failure, etc.)
 * 
 * In the event of failure, addrloc, sizeloc, and dest will be left in an 
 *   undefined state
 */
int srec_parse(char *str, int *addrloc, int *sizeloc, void *dest)
{
  int linelength;
  int size;
  int byte;
  unsigned char checksum = 0;
  int i;
  int addr;
  int addrsize;

  if (str[0] != 'S' && str[0] != 's')
    return -1; //not an s-record!

  linelength = strlen(str);

  if (linelength < 8)
    return -1; //it's not big enough!

  switch (str[1]) {
    case '0':
    case '1':
    case '5':
    case '9':
      addr = readhexn(str+4, 2);
      addrsize = 2; //data starts at str[8]
      break;
    case '2':
    case '8':
      addr = readhexn(str+4, 3);
      addrsize = 3; //data starts at str[10]
      break;
    case '3':
    case '7':
      addr = readhexn(str+4, 4);
      addrsize = 4; //data starts at str[12]
      break;
    default:
      return -1; //unrecognized S-record type
      break;
  }
  if (addr < 0)
    return -1;
  *addrloc = addr;

  if ((size = readhexbyte(str+2)) < 0)
    return -1; //malformed count field

  if (2*size != linelength-4)
    return -1; //count field fails to match length of srecord

  //compute the checksum portion for the count field
  checksum += size;

  //compute the checksum portion for the address field
  for (i = 0; i < addrsize; i++) {
    byte = readhexbyte(str+4+2*i);
    if (byte < 0) //redundant
      return -1; //invalid byte found in srecord
//    printf("Adding in byte %x\n", (unsigned char)byte);
    checksum += byte;
  }

  size -= addrsize;
  //size is now number of character pairs remaining (data & checksum fields)
  
  for (i = 0; i < size-1; i++) {
    byte = readhexbyte(str+4+2*addrsize+2*i);
    if (byte < 0)
      return -1; //invalid byte found in srecord
//    printf("Adding in byte %x\n", (unsigned char)byte);
    *(((char *)dest)+i) = (char)byte;
    checksum += byte;
  }

  checksum = ~checksum; //checksum = 255 - checksum

  byte = readhexbyte(str+4+2*addrsize+2*i); //the checksum in the srecord
  if (byte < 0)
    return -1; //invalid byte found in checksum field
 
  if (checksum != (unsigned char)byte)
    return -1; //checksum mismatch!

  //S-record valid

  *sizeloc = size-1; //don't include the checksum

  return (str[1] - '0');  
}


/*
 * Takes any S-record as null-terminated string str
 *
 * Returns 0 if the s-record appears well-formed and the checksum is correct
 *   or -2 on formatting errors or -1 on checksum failure
 */
int srec_verifychecksum(char *str)
{
  int size;
  int byte;
  unsigned char csum = 0;
  int i;

  if (strlen(str) < 10)
    return -2; //it's not big enough!

  if ((size = readhexbyte(str+2)) < 0)
    return -2; //malformed count field

  if (size < 3)
    return -2; //srec must contain at least a 2-byte addr and the checksum

  size *= 2; //size is now number of ASCII chars remaining
  
  for (i = 0; i < size; i += 2) {
    byte = readhexbyte(str+2+i);
    if (byte < 0)
      return -2; //invalid byte found in srecord
//    printf("Adding in byte %x\n", (unsigned char)byte);
    csum += byte;
  }

  csum = ~csum;

  byte = readhexbyte(str+2+i); //the checksum in the srecord
  if (byte < 0)
    return -2; //invalid byte found in checksum field

//  printf("Calculated checksum: %x\n", csum);
//  printf("Supplied checksum: %x\n", byte);

  if (csum != (unsigned char)byte)
    return -1; //checksum mismatch!

  return 0;
}

/*
 * Takes an S0 record as null-terminated string str
 * Places at most maxlen-1 bytes of the data field into dest, followed by a 
 *   terminating null character (if maxlen == 0, then this function returns 0
 *   and does nothing)
 * 
 * Returns 0 on success or -1 on error (misformatted record, etc.)
 *
 * Note that this does not verify the checksum
 */
int srec_name(char *str, char *dest, int maxlen)
{
  int size;
  int byte;
  int i;

  if (maxlen == 0)
    return 0; //we won't have room for the null!

  if (str[0] != 'S' && str[0] != 's')
    return -1; //not an s-record!

  if (str[1] != '0')
    return -1; //not an S0 record!

  if (strlen(str) < 10)
    return -1; //it's not big enough!

  if ((size = readhexbyte(str+2)) < 0)
    return -1; //malformed count field

  if (size < 3)
    return -1; //srec must contain at least a 2-byte addr and the checksum

  //address should be 0x0000, but we won't enforce that strictly

  size -= 3; //don't count the two address fields or the checksum

  //size is now the number of bytes of data in this S0 record

  for (i = 0; i < size; i++) {
    byte = readhexbyte(str+8+2*i);
    if (byte < 0)
      return -1; //invalid byte found
    if (i >= maxlen-1) {
      dest[i] = '\0';
      return i+1;
    }
    //else,
    dest[i] = (char)byte;
  }

  dest[i] = '\0';

  return size;
}

/*
 * Takes any S-record as null-terminated string str
 *
 * Returns the address specified in the record or -1 on error (misformatted,
 *   etc.)
 *
 * Note that this does not verify the checksum
 */
int srec_address(char *str)
{
  int size;
  int addr;

  if (str[0] != 'S' && str[0] != 's')
    return -1; //not a valid s-record

  if ((size = readhexbyte(str+2)) < 0)
    return -1; //malformed count field

  if (size < 3)
    return -1; //srec must contain at least a 2-byte addr and the checksum

  switch (str[1]) {
    case '0':
    case '1':
    case '5':
    case '9':
      addr = readhexn(str+4, 2);
      break;
    case '2':
    case '8':
      addr = readhexn(str+4, 3);
      break;
    case '3':
    case '7':
      addr = readhexn(str+4, 4);
      break;
    default:
      return -1; //unrecognized S-record type
      break;
  }

  return addr;
}


