/**************************************************************************/
/* FILE NAME: tpu_pwm.c                       COPYRIGHT (c) MOTOROLA 2002 */
/* VERSION: 1.0                                   All Rights Reserved     */
/*                                                                        */
/* DESCRIPTION: This file shows how to use the standard set-up and header */
/* files for the MPC565. This program sets up the MPC565 TPU              */
/* initializations for the Frequency Meassure Function.                   */
/* This program uses the fixed structure feature of the header files.     */
/*========================================================================*/
/* COMPILER: Diab Data        VERSION: 4.3g                               */
/* AUTHOR: Randy Dees                                                     */
/*                                                                        */
/* HISTORY                                                                */
/* REV      AUTHOR      DATE       DESCRIPTION OF CHANGE                  */
/* ---   -----------  ---------    ---------------------                  */
/* 1.0   R. Dees      17/Oct/02   Initial version of function.
/*                    This version does not include the tpu_pwm_force           */
/**************************************************************************/
#include "tpu_pwm.h"
#include "mpc500_util.h"
#include "mpc500_util.c"
#include "m_usiu.h"        /* Included for the interrupt function */

/*************************************************************/
/*            Begin TPU Initialization                       */
/*************************************************************/

/*******************************************************************************
FUNCTION      : tpu_pwm_init
PURPOSE       : To initialize channels to run the PWM Pulse Width Modulation function. 
                This function initializes the pwm function and sets the 
                initial perios. 

INPUTS NOTES  : This function has 7 parameters:
                 *tpu - This is a pointer to the TPU3 module to use. It is of 
                        type TPU3_tag which is defined in m_tpu3.h.
                 channel - This is the channel number that has the pwm function 
                         assigned to it.
                 priority - This is the priority to assign to the pwm function. 
                         The TPU priority definitions are defined in mpc500_utils.h. 
                         This parameter should be assigned a value of:
                         TPU_PRIORITY_HIGH, TPU_PRIORITY_MIDDLE or TPU_PRIORITY_LOW
                 mode - Continuous or single-shot
                 edge - Rising or falling edge
                 timer - use TCR1 or TCR2
                 wind_sz - the size of the sample window

RETURNS NOTES : none
WARNING       : The channels must be stopped before it is reconfigured. The
                function disables the channels but if they were currently
                being serviced it would continue. The delay for assigning the
                pram pointer may to enough but depends on system loading.
*******************************************************************************/
void tpu_pwm_init(struct TPU3_tag *tpu, UINT8 channel, 
		UINT8 priority, UINT16 period, UINT16 hightime, UINT8 timer)
{
UINT16 channel_control;
UINT16 tbs;
UINT8 hsq;
 
    /* disable channels so they can be configured safely */
    tpu_disable( tpu, channel);
    
    /*  PWM is function 0x3                  */
    tpu_func( tpu, channel, TPU_FUNCTION_PWM);

    /* disable interupts on channels so they can be configured safely */
    tpu_interrupt_disable( tpu, channel );
  
   /* mask off illegal values */
	
    tbs = ( timer & TPU_PWM_TBS_MASK ) << 5;
    
    /* Initialize Parameter RAM                                       */
    channel_control = ( tbs | TPU_PWM_PAC | TPU_PWM_PSC ); 

    tpu->PARM.R[channel][TPU_PWM_CHANEL_CONTROL] = channel_control;
    tpu->PARM.R[channel][TPU_PWM_PWMHI] = hightime;
    tpu->PARM.R[channel][TPU_PWM_PWMPER] = period;
   

/**************************************************/
/*    Configure the Channels.                     */
/**************************************************/
   tpu_hsr(tpu, channel, TPU_PWM_INIT);

    
    /* Enable channel by assigning a priority to them. */
    tpu_enable(tpu, channel, priority);

}  /* End tpu_pwm_init */

/*******************************************************************************
FUNCTION      : tpu_pwm_update
PURPOSE       : To change the size of the sample window.
INPUTS NOTES  : This function has 3 parameters:
                 *tpu - This is a pointer to the TPU3 module to use. It is of
                         type TPU3_tag which is defined in m_tpu3.h
                 channel - This is the channel number 
                 period - This is desired period (in TCR counts)
                 hightime - hightime (in TCR counts) of the PWM waveform
*******************************************************************************/

void tpu_pwm_update(struct TPU3_tag *tpu, UINT8 channel, 
		UINT16 period, UINT16 hightime, UINT8 immediate)
{
UINT32 update_val32;

	update_val32 = ((hightime << 16 ) | (period));

	tpu->PARM.L[channel][TPU_PWM_HIPER32] = update_val32;
	if (immediate)
		{
		tpu_hsr(tpu, channel, TPU_PWM_IMMED_UPDATE);
	 }  
}  /* End tpu_pwm_update */

/*******************************************************************************
FUNCTION      : tpu_pwm_get_hightime
PURPOSE       : Returns the current high time of the PWM.
INPUTS NOTES  : This function has 2 parameters:
                 *tpu - This is a pointer to the TPU3 module to use. It is of
                         type TPU3_tag which is defined in m_tpu3.h
                 channel - This is the number of the master channel
OUTPUT NOTES  : This function returns a a UINT16 value
*******************************************************************************/
UINT16	tpu_pwm_get_hightime(struct TPU3_tag *tpu, UINT8 channel)
{
	UINT16 hightime;
	 
	hightime = tpu->PARM.R[channel][TPU_PWM_PWMHI];
    return (hightime);
}  /* End tpu_pwm_get_hightime */

/*******************************************************************************
FUNCTION      : tpu_pwm_get_pulse
PURPOSE       : Returns the current high time of the PWM.
INPUTS NOTES  : This function has 2 parameters:
                 *tpu - This is a pointer to the TPU3 module to use. It is of
                         type TPU3_tag which is defined in m_tpu3.h
                 channel - This is the number of the master channel
OUTPUT NOTES  : This function returns a a UINT16 value
*******************************************************************************/
UINT16	tpu_pwm_get_period(struct TPU3_tag *tpu, UINT8 channel)
{
	UINT16 period;
	 
	period = tpu->PARM.R[channel][TPU_PWM_PWMPER];
    return (period);
}  /* End tpu_pwm_get_hightime */

/*********************************************************************
 *
 * Copyright:
 *	MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
 * 
 *  Motorola assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************/

