/**
 * @brief Simple driver for the SIOP function of the MPC5xx TPU
 *
 * The SIOP function uses 1-3 TPU channels to form a uni- or bidirectional
 *   synchronous serial port.  A common application would be to interface to
 *   SPI-speaking devices.  Note that the TPU function assumes that it is the
 *   master device (driving the clock line).
 *
 * This TPU function uses a clock channel and optionally either or both of a
 *   data input and data output channel.  All functions defined here are to be
 *   called with the number of the clock channel (say, x).  If present, the
 *   output channel must be x+1, and if present the input channel must be x-1.
 *   No initialization or programming of any kind for these secondary channels
 *   need be done.
 *
 * A single shift register is used by the SIOP function.  The direction of the
 *   shift can be selected.  Incoming data (or zeroes, if no input pin is used)
 *   are shifted in one side, and outgoing data are shifted out the other (to
 *   the output pin, or into oblivion if no output pin is used).
 *
 * References: MPC565/MPC566 User Manual, section C.19
 *             tpu_uart.{c,h} and mpc500_util.{c,h} in libMotorola
 *
 * Copyright (C) 2005 Michael Dille.  Provided under the terms of the
 * GNU General Public License, version 2.
 *
 * This is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * @author Michael Dille
 * @date 6/2005
 * @version $Id: tpu_siop.c,v 1.4 2005/06/09 04:11:49 mdille3 Exp $
 */

#include "tpu_siop.h"
#include "mpc500_util.h"

/**
 * @brief Initialize a TPU channel to execute the SIOP function
 *
 * @param tpu A pointer to the TPU structure corresponding to the TPU containing
 *              the desired channel
 * @param channel The number [0,15] of the channel
 * @param priority The priority to assign to this channel.  Should be one of:
 *                   TPU_PRIORITY_HIGH, TPU_PRIORITY_MIDDLE, TPU_PRIORITY_LOW
 * @param mode The operating mode to use for this channel.  Should be one of:
 *               TPU_SIOP_CLOCK_ONLY, TPU_SIOP_TX_ONLY, TPU_SIOP_RX_ONLY,
 *               TPU_SIOP_BIDIRECTIONAL
 * @param polarity The clock polarity to use.  Should be one of:
 *                   TPU_SIOP_NEGEDGE_CLOCK (data valid on falling edge)
 *                   TPU_SIOP_POSEDGE_CLOCK (data valid on rising edge)
 * @param direction The shift direction of SIOP data.  Should be one of:
 *               TPU_SIOP_LSB_FIRST (data shifted out lowest bits, into highest)
 *               TPU_SIOP_MSB_FIRST (data shifted out highest bits, into lowest)
 * @param baud_rate The clock rate to use.  This is the number of TCR1 counts
 *                    in a half-period of the clock (number of TCR1 counts
 *                    during each of the high/low portions).  The range is from
 *                    1 to 0x8000, though the practical minimum is likely to be
 *                    in the hundreds.
 * @param size The number [1,16] of bits per transfer.
 * @param data The first data to be placed in the shift register for output.
 *               May be from 1 to 16 bits in length, justified correctly for the
 *               supplied direction.  Ignored if the mode does not include an
 *               output.  Loaded but not immediately transferred (and thus may
 *               be changed before the first transfer) if the startmode
 *               specifies a deferred start.
 * @param startmode Whether or not to begin the first transfer.  Should be one
 *                    of:
 *                    TPU_SIOP_START (first transfer begins immediately)
 *                    TPU_SIOP_STOP (first transfer occurs when later activated)
 * @param nointerrupt_interrupt Completion interrupts will be enabled on this
 *                                channel if this value is nonzero.
 */
void tpu_siop_init(struct TPU3_tag *tpu, UINT8 channel, UINT8 priority,
                   UINT8 mode, UINT16 polarity, UINT8 direction,
                   UINT16 baud_rate, UINT8 size, UINT16 data,
                   UINT8 startmode, UINT8 nointerrupt_interrupt)
{
  int outchan = (channel + 1) & 0xF;
  int inchan = (channel - 1) & 0xF;

  /* disable the appropriate channels so they can be configured safely */
  tpu_disable(tpu, channel);
  if (mode == TPU_SIOP_TX_ONLY || mode == TPU_SIOP_BIDIRECTIONAL)
    tpu_disable(tpu, outchan);
  if (mode == TPU_SIOP_RX_ONLY || mode == TPU_SIOP_BIDIRECTIONAL)
    tpu_disable(tpu, inchan);

  /* select the SIOP function for the channel */
  tpu_func(tpu, channel, TPU_FUNCTION_SIOP);

  /* setup the clock polarity and shift direction */
  tpu->PARM.R[channel][TPU_SIOP_DIRECTION_POLARITY] = \
    ((polarity & TPU_SIOP_CHAN_CONTROL_MASK) << TPU_SIOP_CHAN_CONTROL_SHIFT) |
    ((direction & TPU_SIOP_BIT_D_MASK) << TPU_SIOP_BIT_D_SHIFT);

  /* setup the desired baud rate by configuring the HALF-PERIOD param */
  tpu->PARM.R[channel][TPU_SIOP_HALF_PERIOD] = baud_rate;

  /* setup the transfer size by configuring the XFER_SIZE param */
  tpu->PARM.R[channel][TPU_SIOP_XFER_SIZE] = size & TPU_SIOP_XFER_SIZE_MASK;

  /* write the data to be transmitted to the data register */
  tpu->PARM.R[channel][TPU_SIOP_DATA] = data;

  /* configure the host sequence bits to set the mode */
  tpu_hsq(tpu, channel, mode & TPU_SIOP_MODE_MASK);

  /* clear the interrupt flag */
  tpu_clear_interrupt(tpu, channel);

  /* enable interrupts if requested */
  if (nointerrupt_interrupt) {
    tpu_interrupt_enable(tpu, channel);
  }

  /* Initialize the clock channel and start the transfer if requested */
  tpu_hsr(tpu, channel, startmode & TPU_SIOP_STARTMODE_MASK);

  /* Enable this channel so it actually gets scheduled by the TPU */
  tpu_enable(tpu, channel, priority);
}

/**
 * @brief Place new data in the transmit buffer.
 *
 * Must be called on a TPU channel initialized to execute the SIOP function.
 * Should be called while the TPU is not actively shifting data in or out.
 * If the continue flag is nonzero, calling this function will start a new
 *   shift and clock sequence.
 * This function should really only be called if in TX-only mode, though the
 *   behavior is well-defined in all cases.
 * 
 * @param tpu A pointer to the TPU structure corresponding to the TPU containing
 *              the desired channel
 * @param channel The number [0,15] of the channel
 * @param transmit_data The data to place into the shift register.  The number
 *                        of bits to be shifted is the same as specified when
 *                        the channel was initialized, and the justification (if
 *                        less than 16 bits) must also be the same.
 * @param cont The continue flag (see above description)
 */
void tpu_siop_write_transmit_data(struct TPU3_tag *tpu, UINT8 channel,
                                  UINT16 transmit_data, int cont)
{
  /* clear interrupt status register */
  tpu_clear_interrupt(tpu, channel);

  /* write data register */
  tpu->PARM.R[channel][TPU_SIOP_DATA] = transmit_data;

  /* issue host service request */
  if (cont)
    tpu_hsr(tpu, channel, TPU_SIOP_START);
}

/**
 * @brief Read data from the shift register.
 *
 * Must be called on a TPU channel initialized to execute the SIOP function.
 * Should be called while the TPU is not actively shifting data in or out.
 * If the continue flag is nonzero, calling this function will start a new
 *   shift and clock sequence.
 * This function should really only be called if in RX-only mode, though the
 *   behavior is well-defined in all cases.
 * 
 * @param tpu A pointer to the TPU structure corresponding to the TPU containing
 *              the desired channel
 * @param channel The number [0,15] of the channel
 * @param cont The continue flag (see above description)
 *
 * @return The current contents of the shift register
 */
UINT16 tpu_siop_read_receive_data(struct TPU3_tag *tpu, UINT8 channel,
                                  int cont)
{
  UINT16 receive_data;

  /* clear interrupt status register */
  tpu_clear_interrupt(tpu, channel);

  /* read data register */
  receive_data = tpu->PARM.R[channel][TPU_SIOP_DATA];
  
  /* issue host service request */
  if (cont)
    tpu_hsr(tpu, channel, TPU_SIOP_START);

  return receive_data;
}

/**
 * @brief Read data from the shift register and replace it with new data.
 *
 * Must be called on a TPU channel initialized to execute the SIOP function.
 * Should be called while the TPU is not actively shifting data in or out.
 * If the continue flag is nonzero, calling this function will start a new
 *   shift and clock sequence.
 * This function should really only be called if in bidirectional mode, though
 *   the behavior is well-defined in all cases.
 * 
 * @param tpu A pointer to the TPU structure corresponding to the TPU containing
 *              the desired channel
 * @param channel The number [0,15] of the channel
 * @param transmit_data The data to place into the shift register.  The number
 *                        of bits to be shifted is the same as specified when
 *                        the channel was initialized, and the justification (if
 *                        less than 16 bits) must also be the same.
 * @param cont The continue flag (see above description)
 *
 * @return The current contents of the shift register
 */
UINT16 tpu_siop_service_read_write(struct TPU3_tag *tpu, UINT8 channel,
                                   UINT16 transmit_data, int cont)
{
  UINT16 receive_data;

  /* clear interrupt status register */
  tpu_clear_interrupt(tpu, channel);

  /* read data register */
  receive_data = tpu->PARM.R[channel][TPU_SIOP_DATA];
  
  /* write data register */
  tpu->PARM.R[channel][TPU_SIOP_DATA] = transmit_data;

  /* issue host service request */
  if (cont)
    tpu_hsr(tpu, channel, TPU_SIOP_START);

  return receive_data;
}

/**
 * @brief Starts a new transfer
 *
 * Must be called on a TPU channel initialized to execute the SIOP function.
 * Should be called while the TPU is not actively shifting data in or out.
 * This will immediately commence a transfer using the settings specified when
 *   the channel was initialized.
 * This function can be used to begin a transfer that was deferred by supplying
 *   TPU_SIOP_STOP as the start mode during initialization or a zero continue
 *   flag to one of the register access functions.
 *
 * @param tpu A pointer to the TPU structure corresponding to the TPU containing
 *              the desired channel
 * @param channel The number [0,15] of the channel
 */
void tpu_siop_start_transfer(struct TPU3_tag *tpu, UINT8 channel)
{
  /* Initialize the clock channel and start the transfer */
  tpu_hsr(tpu, channel, TPU_SIOP_START);
}

/**
 * Debugging function for the SIOP function.  Dumps the contents of parameter
 *   RAM
 */
#if 0
void tpu_siop_debug(struct TPU3_tag *tpu, int channel)
{
  int i;

  for (i = 0; i < 8; i++) {
    printf("param[%d] = %d\n", i, tpu->PARM.R[channel][i]);
  }
}
#endif
 
