// $Id: C3F_flash.c,v 1.1 2005/02/17 19:56:05 garthz Exp $ 
//
// C3F_flash.c : interface to the Motorola C3F driver
//
// The Motorola driver is supplied in binary form only.  This uses the
// driver to program the internal flash.  This code is partly derived
// from a Motorola example.

typedef  unsigned char BOOL; // fixup for gmd_C3F.h, since gmd_type.h conflicts with mpc566.h

#include "mpc566evb.h"
#include <Motorola/mpc566.h>
#include <Motorola/gmd_C3F.h>
#include <Motorola/gmd_internal.h>

#define BDMFLAGS 0  // Not enter into BDM after each operation, leave interrupts disabled

// The Motorola-supplied ELF files are compiled for a "far" call, and ld will complain
// about truncating to a 24 bit relative relocation unless declared as follows:

extern C3F_RESULT BlankCheck() __attribute__ ((longcall));
extern C3F_RESULT FlashErase () __attribute__ ((longcall));
extern C3F_RESULT FlashVerify () __attribute__ ((longcall));
extern C3F_RESULT FlashProgram () __attribute__ ((longcall));
extern C3F_RESULT FlashInit() __attribute__ ((longcall));
extern C3F_RESULT FlashCheckShadow() __attribute__ ((longcall));

/****************************************************************/
// A null callback for the driver to poll.
static void CallBack( void )
{
}

/****************************************************************/
// Internal function to set up the flash registers and determine the
// array address.  Takes a pointer to variable in which to return the
// array base address.  Returns an error code.

static C3F_RESULT C3F_setup( UINT32 *arrayBase_ptr )
{
  C3F_RESULT returnCode;
  UINT32 m;             // module index

  // call FlashInit, which will return the address of the flash array
  // in the location pointed to by arrayBase_ptr.
  
  returnCode = (*FlashInit) ( BDMFLAGS, arrayBase_ptr );

  // return any error
  if (returnCode != C3F_OK) return( returnCode );
    
  // Setup flash map and partitions

  // Set up the various flash module control and configuration register fields
  UC3F_A.UC3FMCR.B.SUPV       = 0;  // Set desired bits to 1 to enable corresponding large block as supervisor space
  UC3F_A.UC3FMCR.B.DATA       = 0;  // Set desired bits to 1 to enable corresponding large block as data space
  UC3F_A.UC3FMCR.B.PROTECT    = 0;  // Set desired bits to 1 to erase and write protect corresponding large block
  UC3F_A.UC3FMCRE.B.SBEN      = 0;  // Set desired bits to 1 to enable corresponding small block

  // The following variables have no effect if the corresponding sb_enable[x] bits are not set
  UC3F_A.UC3FMCRE.B.SBSUPV    = 0;  // Set desired bits to 1 to enable corresponding small block as supervisor space
  UC3F_A.UC3FMCRE.B.SBDATA    = 0;  // Set desired bits to 1 to enable corresponding small block as data space
  UC3F_A.UC3FMCRE.B.SBPROTECT = 0;  // Set desired bits to 1 to erase and write protect corresponding small block

  // Set up the various flash module control and configuration register fields
  UC3F_B.UC3FMCR.B.SUPV       = 0;  // Set desired bits to 1 to enable corresponding large block as supervisor space
  UC3F_B.UC3FMCR.B.DATA       = 0;  // Set desired bits to 1 to enable corresponding large block as data space
  UC3F_B.UC3FMCR.B.PROTECT    = 0;  // Set desired bits to 1 to erase and write protect corresponding large block
  UC3F_B.UC3FMCRE.B.SBEN      = 0;  // Set desired bits to 1 to enable corresponding small block

  // The following variables have no effect if the corresponding sb_enable[x] bits are not set
  UC3F_B.UC3FMCRE.B.SBSUPV    = 0;  // Set desired bits to 1 to enable corresponding small block as supervisor space
  UC3F_B.UC3FMCRE.B.SBDATA    = 0;  // Set desired bits to 1 to enable corresponding small block as data space
  UC3F_B.UC3FMCRE.B.SBPROTECT = 0;  // Set desired bits to 1 to erase and write protect corresponding small block

  return C3F_OK;
}
/****************************************************************/
// Internal function to set up shadow row configuration.
  // Set shadow row configuration.  "This function should be called
  // before FlashProgram, BlankCheck, FlashVerify, or CheckSum
  // operation, even if shadow memory will not be used. This is
  // important to prevent unintended shadow row operations."

static C3F_RESULT C3F_set_shadow_config( UINT32 arrayBase )
{

  return (*FlashCheckShadow)(  BDMFLAGS,
			       0, // select main rather than shadow memory
			       0, // dest, irrelevant
			       0, // size, irrelevant
			       arrayBase );
}

/****************************************************************/
UINT32 C3F_flash_erase(void)
{
  UINT8 enabledBlocks[2];   // Enabled array blocks          
  UINT8 enabledSBlocks[2];  // Enabled array small blocks    
  UINT32 arrayBase;         // Store starting address of C3F     
  C3F_RESULT returnCode;    // Return code from each GMD function.                                                  
  UINT32 compareAddress;    // Result from BlankCheck().
  UINT32 compareData;       // Result from BlankCheck().

  // Initialize part.
  returnCode = C3F_setup( &arrayBase );
  if ( returnCode != C3F_OK ) return( returnCode );

  // Erase all C3F main arrays and shadow rows.
  enabledBlocks[0]  = enabledBlocks[1]  = 0xFF;   // Select all blocks in module m to erase    
  enabledSBlocks[0] = enabledSBlocks[1] = 0x00;   // De-select all small blocks in module      

  returnCode = (*FlashErase)( BDMFLAGS,
			      CallBack,
			      enabledBlocks,
			      enabledSBlocks,
			      arrayBase );

  if ( returnCode != C3F_OK ) return( returnCode );


  // Blank check main array.

  // Set shadow row configuration.  "This function should be called
  // before FlashProgram, BlankCheck, FlashVerify, or CheckSum
  // operation, even if shadow memory will not be used. This is
  // important to prevent unintended shadow row operations."

  returnCode = C3F_set_shadow_config( arrayBase );
  if ( returnCode != C3F_OK ) return( returnCode );

  returnCode = (*BlankCheck)(    BDMFLAGS,
				 CallBack,
				 0, // dest
				 2 * C3F_MODULE_SIZE, // size
				 &compareAddress,
				 &compareData,
				 arrayBase);

  return( returnCode );
}
/****************************************************************/
// Write the given data buffer into the flash, starting at address 0.
// The buffer must be 4 byte aligned.  The length is in bytes.

UINT32 C3F_flash_write( UINT32 *dest, UINT32 *source, UINT32 source_length )
{
  UINT32 arrayBase;         // Store starting address of C3F     
  C3F_RESULT returnCode;    // Return code from each GMD function.                                                  
  UINT32 compareAddress;    // Result from FlashVerify().
  UINT32 compareData;       // Result from FlashVerify().
  UINT32 compareSourceData; // Result from FlashVerify().

  // Initialize part.
  returnCode = C3F_setup( &arrayBase );
  if ( returnCode != C3F_OK ) return( returnCode );
    
  returnCode = C3F_set_shadow_config( arrayBase );
  if ( returnCode != C3F_OK ) return( returnCode );

  returnCode = (*FlashProgram)(  BDMFLAGS,
				 CallBack,
				 (UINT32) dest,
				 source_length,
				 (UINT32) source,
				 arrayBase);

  if ( returnCode != C3F_OK ) return( returnCode );


  // Verify array data

  returnCode = C3F_set_shadow_config( arrayBase );
  if ( returnCode != C3F_OK ) return( returnCode );
    
  returnCode = (*FlashVerify) ( BDMFLAGS,
				CallBack,
				(UINT32) dest,
				source_length,
				(UINT32) source,
				&compareAddress,
				&compareData,
				&compareSourceData,
				arrayBase );

  if (returnCode != C3F_OK) return( returnCode );

  return C3F_set_shadow_config( arrayBase );
}
