// Initialization code for MPC566 hardware.  This is only a fraction
// of the total internal state, much of the configuration is left 
// in the default state.
//
// This file was based almost entirely on phyCORE_MPC565.c
//
// This code assume that the internal hardware is all mapped at the
// default address.
//
// These routines assume an MPC566EVB board (manufactured by Motorola)
//
// External Flash:
//   2MByte = 1x Am29BDD160G (each 2Mbyte = 16Mbit = 512k x 32)
//   Top 1Mbyte is occupied by MPC566EVB dBug monitor
//   Controlled by /CS0 or /CS2 (/CS0 by default)
//     MAP_SW #3 on ==> /CS0 used (default), MAP_SW #4 ==> /CS2 used
//
// External SRAM:
//   1Mbyte = 2x Cypress ... (each 512Kbyte = 128K x 32)
//   Controlled by /CS0 or /CS1 (/CS1 by default)
//     MAP_SW #1 on ==> /CS0 used, MAP_SW #2 ==> /CS1 used (default)
//
// Internal memory map:
// 0000 0000   internal FLASH A 512K
// 0008 0000   internal FLASH B 512K
// 0010 0000   reserved
// 002F 8000   beginning of internal I/O and reserved space
// 0030 1000   DPTRAM_C 4K
// 0030 2000   DPTRAM_AB 6K
// 003F 7000   CALRAM_B 4K
// 003F 8000   CALRAM_A 32K
//
// external address space begins:  0040 0000
// I map the external SRAM at:     0100 0000
// The dBUG monitor occupies:      0110 0000 - 0120 0000
// I map the standard FLASH at:    0200 0000

// The reset exception location is 0x00000100


/****************************************************************/
#include "mpc566evb.h"

// Use the Motorola hardware definition, which provides a lot of unions 
// to represent the registers.
#include <Motorola/mpc566.h>

//These are necessary for serial initialization and are declared in
//serial.c (libnewlib5xx / libmpc5xx)
extern struct QSMCM_tag *serial_defqsmcm;
extern int serial_defsci;


/****************************************************************/
// Initialize some hardware.  This should only be called once.  Note
// that this is called before the .data section is initialized, so no
// references to initialized global variables are allowed.  This can
// be checked by compiling to assembly; there should be no references
// to .data, .sdata, or .sdata2 in this function.  This is necessary
// because the memory controller registers must be configured before
// the memory can be initialized!

void mpc566evb_init(void)
{
  // This sequence was modified from the Motorola code in mpc500.c.

  // Disable watchdog timer. Once disabled, it cannot be re-enabled.
  // SYPCR is the System Protection Control Register, described in
  // section 6.15.3.1 of the MPC565 User's Manual.

  // USIU.SYPCR.R = 0xffffff03;
  USIU.SYPCR.R = 0x0000ff00; // write to SYPCR: disable bus monitor, disable watchdog, etc. (0x002fc004)

  // SIUMCR is the System Interface Unit Module Configuration Register, described in 
  // Section 6.14.1.1.
  // USIU.SIUMCR.R = 0x00000000; // Expanded mode, normal pin options
  USIU.SIUMCR.R = 0x00020000; // write to SIUMCR: GPC=01 (see Table 6-9), relates to pins SPGIOC6 and SPGIOC7 (0x002fc000)

  // PDMCR is the Pads Module Configuration Register, described in Section 2.2.
  USIU.PDMCR.R = 0xf0000000; // write to PDMCR: Normal slew rate with no pull ups on Pads (0x002fc03c)

  // SCCRK is the the System Clock Control Register Key, see Table 8-8
  // SCCR is the the System Clock Control Register, Section 8.11.1, Table 8-9
  // The following lines were commented out in the Motorola code:

  // USIU.SCCRK.R = 0x55ccaa33; // Write SCCRK with unlock key (0x002fc380)
  // USIU.SCCR.R  = 0x01300100; // Write SCCR: initialise clocks with external 4MHz osc. (0x002fc280)

  //                0x81213f00 is the actual default value of SCCR in my hardware

  // PLPRCRK is the PLL, Low-Power and Reset Control Register Key, see Table 8-8.
  USIU.PLPRCRK.R = 0x55ccaa33;  // Write PLPRCRK with the unlock key (*0x002fc384 =0x55ccaa33)

  // PLPRCR is the PLL, Low-Power and Reset Control Register, see Section 8.11.2.

#if SYSTEM_CLOCK==20000000
  // Set up a 20 MHz system clock:
  USIU.PLPRCR.R = 0x0043d000; // write PLPRCR: 4x PLL operation on normal power mode (write to 0x002fc284)

#elif SYSTEM_CLOCK==40000000
  // Set up a 40 MHz system clock:
  USIU.PLPRCR.R = 0x0093d000; // write PLPRCR: 10x PLL operation on normal power mode (write to 0x002fc284)

#elif SYSTEM_CLOCK==56000000
  // 56MHz is an optional feature of the MPC566.
  USIU.PLPRCR = 0x00d3d000; // write PLPRCR: 14x PLL operation on normal power mode (write to 0x002fc284)

#else
#error invalid SYSTEM_CLOCK
#endif

  // TBSCRK is the Time Base Status Control Register Key, see Table 8-8
  USIU.TBSCRK.R = 0x55ccaa33; // Write TBSCRK with the unlock key (*0x002fc300 = 0x55ccaa33)

  // TBSCR is the Time Base Status Control Register, see Section 6.14.4.4
  USIU.TBSCR.R = 0x0001; // Write TBSCR: Enable TB and decrementer  (0x002fc200)

  // UMCR is the UIMB Module Configuration Register, see Section 12.5.1
  UIMB.UMCR.B.HSPEED = 0; // Write UMCR: clear bit HSPEED to IMB at full clock speed  (0x00307f80)

  // Initialize the Base and Option Registers which control chip
  // select generation, and hence the run-time memory map.  See page
  // 10-36 of the MPC566 Reference Manual for explanation of BR and OR.

  // The MPC566EVB datasheet suggests placing the sram base at 0xFFF00000
  // with corresponding register values BR1 = 0xFFF0_0003, OR1 = 0xFFF0_0000
  // We instead use a base of 0x01000000
  // 
  // BR1:
  //    0100 xxxx   : base address at 0x0100 0000
  //    lower bits  : address type 0, 32 bit port, normal setup time, read/write, 
  //                  burst inhibited, ..., registers valid
  // OR1: 0xff00 0000
  //    ff00 xxxx   : address mask; compare only the MSB 8 bits to map a 16Mb region at the base address
  //    lower bits  : address type 0, ...memory timing modes, etc....
  USIU.OR1.R = 0xff000000; // OR1 (0x002fc10c)
  USIU.BR1.R = 0x01000003; // BR1 (0x002fc108)

  // The MPC566EVB datasheet suggests placing the flash base at 0x00800000
  // with corresponding register values BR0 = 0x0080_003, OR0 = 0xFFE0_0030
  // We instead use a base of 0x02000000
  //
  // BR0:
  //    0200 xxxx   : base address at 0x0200 0000
  //    lower bits  : address type 0, 32 bit port, normal setup time, read/write,
  //                   burst inhibited, ..., register valid
  // OR0:
  //    ff00 xxxx   : address mask; compare only the MSB 8 bits to map a 16Mb region at the base address
  //    lower bits  : address type 0, 3 wait states
  USIU.OR0.R = 0xff000030; // OR0 (0x002fc108)
  USIU.BR0.R = 0x02000003; // BR0 (0x002fc104)

  // The MPC566EVB datahseet suggests placing the ethernet base at 0x01000000
  // with corresponding register values BR3 = 0x0100_0807, OR3 = 0xFFFF_80F0
  //
  // We instead use a base of 0x03000000
  // This is hardcoded in the mpc566evb.h:  #define MPC566EVB_ETHERNET_BASE  0x03000000
  //
  // BR3:
  //    0300 xxxx   : base address at 0x0300 0000
  //    lower bits  : address type 0, 16 bit port, normal setup time, read/write, external transfer acknowledge, burst inhibit, 16-bit port
  // OR3:
  //    ff00 xxxx   : address mask; compare only the MSB 8 bits to map a 16Mb region at the base address
  //    lower bits  : address type 0, 15 wait states
  USIU.OR3.R = 0xFF0080F0; //OR3 (0x2fc11c)
  USIU.BR3.R = 0x03000807; //BR3 (0x2fc118)

  /****************************************************************/
  // This sequence was copied from the Motorola code in mpc500.c, and
  // translated a bit for GCC/GAS.  It assumes that the internal registers
  // are based the default location (zero).

  // Special Purpose Register 158 is the ICTRL I-Bus Support Control Register, see Section 22.6.6, Tables 22-21
  asm ( "mfspr %r3,158" );      // Put PowerPC core in non-serialized (normal) mode with no show cycles
  asm ( "ori %r3,%r3,0x0007" ); // See Table 22-22 
  asm ( "mtspr 158,%r3" );

  // Special Purpose Register 560 is the BBCMCR BBC Module Configuration Register, see Section 4.7.2.1, Table 4-4
  asm ( "mfspr %r3,560" );      /* Enable burst buffer */
  asm ( "ori %r3,%r3,0x2000" );
  asm ( "mtspr 560,%r3" );

  // writes to the BBCMCR should be followed with an ISYNC to hold off
  // any instructions until the register command completes.
  asm (" isync");  

  // Enable floating point and make machine check exception recoverable.
  // The layout of the MSR is defined in Section 2.3.1 of the PowerPC Programming Environments Manual.
  asm ( "mfmsr %r3" );             // Read the Machine State Register into R3
  asm ( "ori %r3,%r3,0x3000" );    // set bits FP and ME: float-point enable, machine check enable ("Machine check exceptions are enabled")
  asm ( "mtmsr %r3" );             // and store back in the MSR    

  /****************************************************************/

  //The MPC566EVB does not have any DIO ports dedicated to user LEDs
  //Nevertheless, it might be convenient to attach one to D0

  MIOS14.MPIOSM32DR.B.D0 = 1;     // drive MPIO0 high, LED will start out off
  MIOS14.MPIOSM32DDR.B.DDR0 = 1;  // configure MPIO0 for output
}

/****************************************************************/
// Control for D0, which can be attached to an indicator LED.
void mpc566evb_set_LED(int flag)
{
  // Modify the port data register.
  if ( flag )   MIOS14.MPIOSM32DR.B.D0 = 0;  // drive MPIO0 low, LED will light
  else          MIOS14.MPIOSM32DR.B.D0 = 1;  // drive MPIO0 high
}


/****************************************************************/
// Initialize the serial port.
//  qsmcm = 0 for QSMCM_A, 1 for QSMCM_B
//  sci = 0 for SCI1, 1 for SCI2

#define B_115200       (SYSTEM_CLOCK / 115200 / 32 + 1)
#define B_57600       (SYSTEM_CLOCK / 57600 / 32)
#define B_19200       (SYSTEM_CLOCK / 19200 / 32)
#define B_9600       (SYSTEM_CLOCK / 9600 / 32)

void mpc566evb_init_serial( int qsmcm, int sci )
{
  struct QSMCM_tag *qsmcm_ptr;

  // QSMCM is the Queued Serial Multi-Channel Module (there are two QSMCM units on the MPC565).
  // Each QSMCM has a QSPI, SCI1, and SCI2
  // QSPI is the queued serial peripheral interface
  // SCI1 and SCI2 are the serial communications interfaces
  // Table 14-3 summarizes the register map.

  // SCC1R0 is the SCI1 Control Register 0 at 0x00305008, see Table 14-27
  // SCC1R1 is the SCI1 Control Register 1 at 0x0030500a, see Table 14-28
  // SC1SR  is the SCI1 Status Register    at 0x0030500c, see Table 14-29
  // SC1DR  is the SCI1 Data Register      at 0x0030500e, see Table 14-30

  // Set the baud rate and configure the SCI. Valid divisors range from 1 to 8191. See Sections 14.8.7.3 and 14.8.2.
#define SERIAL_CONFIG 0x000c // Transmit & Receive Enabled, no parity, no interrupts
  qsmcm_ptr = ((qsmcm == 0) ? &QSMCM_A : &QSMCM_B); 
  
  if ( sci == 0 ) {
    qsmcm_ptr->SCC1R0.R = B_57600;          // write SCC1R0: set the baud rate to f_sys / (32 * SCxBR)  
    qsmcm_ptr->SCC1R1.R = SERIAL_CONFIG;    // write SCC1R1: Transmit & Receive Enabled, no parity, no interrupts
  } else {
    qsmcm_ptr->SCC2R0.R = B_57600;          // write SCC2R0: set the baud rate to f_sys / (32 * SCxBR)  
    qsmcm_ptr->SCC2R1.R = SERIAL_CONFIG;    // write SCC2R1: Transmit & Receive Enabled, no parity, no interrupts
  }
#undef SERIAL_CONFIG

  if (serial_defsci < 0) {
    //These haven't been set (yet) by the main application, be safe and do it
    //Set the default port for putchar(), et al.
    serial_defqsmcm = &QSMCM_A; //Use QSMCM module A (rather than B)
    serial_defsci = 0; //Use port SCI1
  }
}

/****************************************************************/
