//Interface for writing to the external flash chips on the MPC566EVB
//board.

//Mostly written by Garth Zeglin
//Implements the standard JEDEC flash EEPROM write algorithm
//  to write command sequences to a single 32-bit flash rom

#include "mpc566evb.h"
#include "mpc566evb_flash.h"

// The command codes are generally written to two addresses within the
// flash, 0x555 and 0x2aa.  However, the bottom two bits of the
// address bus don't go to the flash chips, since they select the byte
// within the 32-bit word presented by the two chips combined.
// Referring to these addresses via a unsigned int pointer effectively
// shifts them up two bits.

static volatile int *flash_base = (unsigned int *) MPC566EVB_STD_FLASH_BASE;

static unsigned short blink_count;  // for flashing the LED

// These are the starting addresses of the sectors comprising the first
// megabyte of (half of) the Am29BDD160G flash memory (p. 32 of specsheet)
// This first half can be safely erased, while the upper half contains the
// dBUG monitor.
// For simplicity, it is terminated by 0xffffffff
static unsigned int flash_sectors[] = {
  0x0, 0x800, 0x1000, 0x1800, 0x2000, 0x2800, 0x3000, 0x3800, 0x4000,
  0x8000, 0xc000, 0x10000, 0x14000, 0x18000, 0x1c000, 0x20000, 0x24000,
  0x28000, 0x2c000, 0x30000, 0x34000, 0x38000, 0x3c000, 0xffffffff
};

/****************************************************************/
// Note the lack of error reporting.  I assume it is easier to run a
// separate verification than deal with propagating an error return
// value.

void mpc566evb_flash_write_word(unsigned int *address, unsigned int value)
{
  volatile unsigned int *flash_ptr = address;
  unsigned int status;
  unsigned short timeout = 0x1000;

  flash_base[ 0x555 ] = 0x00AA;        /* program command sequence */
  flash_base[ 0x2aa ] = 0x0055;
  flash_base[ 0x555 ] = 0x00A0;

  *flash_ptr = value; // program a word into the chip
 
  do {
    status = *flash_ptr;   // read the programming status

    // Exit the loop when the chip is done, either by indicating
    // completion on DQ7, or by indicating timeout on DQ5.

    if ( ( ((status & 0x00000080) == (value & 0x00000080)) ||   //is complete?
	   ((status & 0x00000020) != 0) ) )
      break;

    // blink light while programming
    blink_count++;
    mpc566evb_set_LED( blink_count & 0x4000 );

  } while (timeout-- > 0);

  // If the write operation failed, reset the chip
  if (( status & 0x00000020 ) != 0 || timeout == 0)
    mpc566evb_flash_reset();
  
}

// Erase the entire "user" portion of the chip (the part that doesn't contain
// the dBUG monitor)
void mpc566evb_flash_erase_all_user(void)
{
  int i;

  for (i = 0; flash_sectors[i] != 0xffffffff; i++) {
    mpc566evb_flash_erase_sector((unsigned int*)(&flash_base[flash_sectors[i]]));
  }
}

// Erase the entire chip, which takes a while.
void mpc566evb_flash_erase_all(void)
{
  // Chip Erase sequence.
  flash_base[ 0x555 ] = 0x00AA;
  flash_base[ 0x2aa ] = 0x0055;
  flash_base[ 0x555 ] = 0x0080;
  flash_base[ 0x555 ] = 0x00AA;
  flash_base[ 0x2aa ] = 0x0055;
  flash_base[ 0x555 ] = 0x0010;    

  // Poll DQ7 on the chip until it goes high to indicate a successful erase.
  // "During chip erase, a valid address is any non-protected sector address."
  // In principle, this could also examine DQ5 to monitor for failure.

  while((*flash_base & 0x00000080) != 0x00000080) 
    {
      // blink light while erasing
      blink_count++;
      mpc566evb_set_LED( blink_count & 0x8000);
    }

  mpc566evb_flash_reset();
}

// Erase a single sector, given an address within that sector.
extern void mpc566evb_flash_erase_sector(unsigned int *address)
{
  // Sector Erase sequence.
  flash_base[ 0x555 ] = 0x00AA;
  flash_base[ 0x2aa ] = 0x0055;
  flash_base[ 0x555 ] = 0x0080;
  flash_base[ 0x555 ] = 0x00AA;
  flash_base[ 0x2aa ] = 0x0055;
  *address = 0x0030;    

  // Poll DQ7 on the chip until it goes high to indicate a successful erase.
  // "During chip erase, a valid address is any non-protected sector address."
  // In principle, this could also examine DQ5 to monitor for failure.

  while((*address & 0x00000080) != 0x00000080) 
    {
      // blink light while erasing
      blink_count++;
      mpc566evb_set_LED( blink_count & 0x8000);
    }

  mpc566evb_flash_reset();
}

void mpc566evb_flash_reset(void)
{
  // Reset sequence.  This returns the internal state machine to read access.
  flash_base[ 0x555 ] = 0x00AA;
  flash_base[ 0x2aa ] = 0x0055;
  flash_base[ 0x555 ] = 0x00F0;
}

unsigned int mpc566evb_flash_read_manuf_ID(void)
{
  unsigned int manuf_ID;
  // read Manufacturer ID command sequence
  flash_base[ 0x555 ] = 0x00AA;
  flash_base[ 0x2aa ] = 0x0055;
  flash_base[ 0x555 ] = 0x0090;
  manuf_ID = flash_base[ 0 ];
  
  mpc566evb_flash_reset();

  return manuf_ID;
}    
unsigned int mpc566evb_flash_read_device_ID(void)
{
  unsigned int device_ID;
  // read Device ID command sequence
  flash_base[ 0x555 ] = 0x00AA;
  flash_base[ 0x2aa ] = 0x0055;
  flash_base[ 0x555 ] = 0x0090;
  device_ID = flash_base[ 1 ];

  mpc566evb_flash_reset();

  return device_ID;
}

