
#include <stdlib.h>
#include "atomic.h"
#include "interruptutils.h"
#include "iobufferinterrupts.h"

static int writebuf(iobuffer_t *iobuf, char c);
static int readbuf(iobuffer_t *iobuf);
static void outbuf_reader(iobuffer_t *iobuf);
static void inbuf_writer(iobuffer_t *iobuf);
static void iobuffer_null_fn();
static int iobuffer_null_fn2();

int new_iobuffer(iobuffer_t *iobuf)
{
  iobuf->buf = malloc(iobuf->size+1); //+1 b/c one wasted when buffer full
  if (iobuf->buf == NULL)
    return -1; //malloc error

  iobuf->inpos = iobuf->outpos = -1;
  iobuf->readerlock = 0;
  iobuf->intsactive = 0;

  iobuf->readfn = (char(*)())iobuffer_null_fn2;
  iobuf->writefn = (void(*)(char))iobuffer_null_fn;
  iobuf->readyfn = (int(*)())iobuffer_null_fn2;
  iobuf->ackfn = iobuffer_null_fn;
  iobuf->enableintsfn = iobuffer_null_fn;
  iobuf->disableintsfn = iobuffer_null_fn;

  return 0;
}

int delete_iobuffer(iobuffer_t *iobuf)
{
  if (iobuf->intsactive)
    disable_iobuffer_interrupts(iobuf);

  //sanity check that can't hurt
  if (iobuf->buf == NULL)
    return -1;

  free(iobuf->buf);
  iobuf->buf = NULL;

  return 0;
}


//assumption: no more than one writer will be executing simultaneously for a
// given buffer
static int writebuf(iobuffer_t *iobuf, char c)
{
  int loc;

  loc = (iobuf->inpos+1) % iobuf->size;

  if (loc == iobuf->outpos) {
    //Adding one more char would fill the output buffer completely (and we
    //  wouldn't be able to tell whether it was empty or full).  Bomb out.
    return -1;
  }

  iobuf->buf[loc] = c;
  iobuf->inpos = loc;

  return 0;
}

//assumption: no more than one reader will be executing simultaneously for a
// given buffer
static int readbuf(iobuffer_t *iobuf)
{
  if (iobuf->outpos == iobuf->inpos)
    return -1; //buffer is empty

  iobuf->outpos++;
  iobuf->outpos %= iobuf->size;

  return iobuf->buf[iobuf->outpos];
}

int iobuf_write(iobuffer_t *iobuf, char c)
{
  if (writebuf(iobuf, c) < 0)
    return -1;

  outbuf_reader(iobuf);

  return 0;
}

int iobuf_read(iobuffer_t *iobuf)
{
  return(readbuf(iobuf));
}

static void outbuf_reader(iobuffer_t *iobuf)
{
  int c;

  if (!atomic_compare_and_swap(&iobuf->readerlock, 0, 1)) {
    //it must be that we are running from an interrupt (otherwise it couldn't
    //  be locked)
    //we're interrupting the same function; let that instance do the work

    //However, we must make sure that we don't immediately come back, never
    //  giving a chance for the actual reader to do its work:
    iobuf->disableintsfn();

    return;
  }

  if (iobuf->readyfn()) {
    //the device is ready for more data
    c = readbuf(iobuf);
    if (c < 0) {
      //no data is available; nothing we can do
      iobuf->readerlock = 0;
      iobuf->disableintsfn();
      return;
    }
    iobuf->writefn(c);
    iobuf->ackfn();
  }

  iobuf->readerlock = 0;
  iobuf->enableintsfn();
}

static void inbuf_writer(iobuffer_t *iobuf)
{
  int c;

  if (iobuf->readyfn()) {
    //the device has more data
    c = iobuf->readfn();
    iobuf->ackfn();
    writebuf(iobuf, c);
  }
}


void register_iobuffer_interrupts(iobuffer_t *iobuf)
{
  if (iobuf->type == IOBUFFER_IN)
    iobuf->inthandler.fn = (void(*)(void*))inbuf_writer;
  else //iobuf->type == IOBUFFER_OUT
    iobuf->inthandler.fn = (void(*)(void*))outbuf_reader;
  iobuf->inthandler.fnarg = iobuf;
  iobuf->intsactive = 1;
  mpc5xx_add_ext_interrupt_handler(&iobuf->inthandler);
}

void unregister_iobuffer_interrupts(iobuffer_t *iobuf)
{
  mpc5xx_remove_ext_interrupt_handler(&iobuf->inthandler);
  iobuf->intsactive = 0;
}

void enable_iobuffer_interrupts(iobuffer_t *iobuf)
{
  iobuf->enableintsfn();
}

void disable_iobuffer_interrupts(iobuffer_t *iobuf)
{
  iobuf->disableintsfn();
}

static void iobuffer_null_fn()
{
  return; //do nothing at all
}

static int iobuffer_null_fn2()
{
  return 0; //not ready or nothing here or whatever
}


