
/* Serial Port I/O Routines
 *
 * Based on a work (C) Copyright 1999 Axiom Manufacturing.  All rights reserved.
 *
 * Modifications (C) 2004 Michael Dille
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <Motorola/mpc555.h>
#include <Motorola/m_usiu.h>

#include "serial.h"

struct QSMCM_tag *serial_defqsmcm;
int serial_defsci = -1;

// reset / clear the watchdog
void ClearWatchdog(){
	USIU.SWSR.R = 0x556C;	// clear the watchdog
	USIU.SWSR.R = 0xAA39;
}

/*----------------------------------------------------------------------------
ReadyToSend
-----------
 Wait for the serial port Status register to indicate the transmiter is
 ready to receive another byte.

 INPUT:		qsmcm_ptr	pointer to control struct for desired QSMCM
		ioport		0= SCI1, 1= SCI2
 OUTPUT:    returns 0 when ready.  NO TIMEOUT is currently implemented
----------------------------------------------------------------------------*/
char ReadyToSend(struct QSMCM_tag *qsmcm_ptr, char portnum){
	short Status;

	do {
		//Wait for transmit byte sent to the shift register
		//(wait for transmit data register empty to go high)
		//See Table 14-25
		if(portnum == 0)
			Status = qsmcm_ptr->SC1SR.B.TDRE;
		else
			Status = qsmcm_ptr->SC2SR.B.TDRE;
		ClearWatchdog();	// feed the dog
	} while(!Status);

	return 0;
}


/*----------------------------------------------------------------------------
putchar2
-------
 Wait for the serial port transmiter to return ready then write a single byte
 to the transmit register.  Uses the default (globally set) QSMCM and SCI port.

 INPUT:		sbyte		Value to write
 OUTPUT:    returns 0 when finished.  NO TIMEOUT is currently implemented
----------------------------------------------------------------------------*/
char putchar2(unsigned char sbyte) {
	struct QSMCM_tag *qsmcm_ptr = serial_defqsmcm; //use the default
	int ioport = serial_defsci; //use the default

	if (ReadyToSend(qsmcm_ptr, ioport))
		return 1; // wait for ready to send
	//Place the byte in the appropriate data register
	//See Table 14-26
	if (ioport == 0)
		qsmcm_ptr->SC1DR.R = (short) sbyte;
	else
		qsmcm_ptr->SC2DR.R = (short) sbyte;
	return 0;
}

/*----------------------------------------------------------------------------
putch
-----
 Send a single byte out the serial port.
 Uses the default (globally set) QSMCM and SCI port.
 If this is the '\n' character then 2 bytes are sent which are the carriage
 return followed by line feed characters.

 INPUT:		ch			Value to write
----------------------------------------------------------------------------*/
void putch(UINT8 ch){

	if (ch == '\n'){
		putchar2(CR);
		putchar2(LF);
	}
	else{
		putchar2(ch);
	}
}

/*----------------------------------------------------------------------------
puts2
----
 Send a string of characters to the serial port.  The string must end in a
 NULL character (0x00).
 Uses the default (globally set) QSMCM and SCI port.

 INPUT:		sptr		pointer to the string
----------------------------------------------------------------------------*/
void puts2(char *sptr){
	while(*sptr){
		putch(*sptr);
		++sptr;
	}
}

/*----------------------------------------------------------------------------
SerialReady
-----------
 returns 0 if any new serial data has been received

 INPUT:		qsmcm_ptr	pointer to control struct for desired QSMCM
		portnum		0= SCI1, 1= SCI2
 OUTPUT:    returns 0 when new data is ready, 1 otherwise.
----------------------------------------------------------------------------*/
char SerialReady(struct QSMCM_tag *qsmcm_ptr, char portnum){
	short Status;

	ClearWatchdog();	// feed the dog
	//Data is ready if the receive data register is full (RDRF flag set)
	//See Table 14-25
	if(portnum == 0)
		Status = qsmcm_ptr->SC1SR.B.RDRF;
	else
		Status = qsmcm_ptr->SC2SR.B.RDRF;

	if(!Status)
		return 1;	// no data ready
	return 0;	// data ready
}

/*----------------------------------------------------------------------------
ReadSerialPort
--------------
 Reads a single byte from the selected serial port

 INPUT:		qsmcm_ptr	pointer to control struct for desired QSMCM
		pnum		0= SCI1, 1= SCI2
 OUTPUT:
 			returns the byte that was received
----------------------------------------------------------------------------*/
char ReadSerialPort(struct QSMCM_tag *qsmcm_ptr, char pnum){
	short Temp1;
	char SByte;

	//Read a byte from the appropriate data register
	//See Table 14-26
	if(pnum == 0)
		Temp1 = qsmcm_ptr->SC1DR.R; 	// read the port
	else
		Temp1 = qsmcm_ptr->SC2DR.R; 	// read the port
	SByte = (unsigned char) (Temp1 & 0x00FF);
	return SByte;
}

/*----------------------------------------------------------------------------
GetSByte
--------
 Wait for a single byte to be received from the serial port then read it.
 Uses the default (globally set) QSMCM and SCI port.

 INPUT: NONE	
 OUTPUT:
 		returns the byte that was received
----------------------------------------------------------------------------*/
UINT8 GetSByte(){
	struct QSMCM_tag *qsmcm_ptr = serial_defqsmcm; //use the default
	int ioport = serial_defsci; //use the default

	// wait for new incoming data
	while(SerialReady(qsmcm_ptr, ioport) == 1);

	return(ReadSerialPort(qsmcm_ptr, ioport));
}
/*----------------------------------------------------------------------------
kbhit
-----
 returns 0 if any new serial data has been received
 (does the same thing as SerialReady(), but 
   uses the default (globally set) QSMCM and SCI port.)

 INPUT:		ioport		0= SCI1, 1= SCI2
 OUTPUT:    returns 0 when new data is ready, 1 otherwise.
----------------------------------------------------------------------------*/
UINT8 kbhit() {
	return(SerialReady(serial_defqsmcm, serial_defsci));
}

