// $Id: TouCAN_driver.c,v 1.1 2005/03/15 22:24:31 garthz Exp $
//
// TouCAN_driver.c : simple driver for a TouCAN module on a MPC5xx microcontroller.
//
// Copyright (C) 2004 Garth Zeglin.  Provided under the terms of the
// GNU General Public License.
//
//****************************************************************/
// Assumptions:
//   TouCAN hardware is on a MPC5xx controller 
//   40MHz CPU clock, 1 MHz or 500 kHZ CAN bus 
//   the interface is polled (no interrupts)
//   buffers are statically allocated as receive or transmit
//   only standard frames are used, no extended frames
//   no "remote frames" are used
//
// This code performs a minimum of error checking and reporting.
//
/****************************************************************/
// Use the Motorola hardware definition, which provides a lot of unions 
// to represent the registers.
#include <Motorola/m_toucan.h>
#include "TouCAN_driver.h"

#ifndef NULL
#define NULL ((void *) 0)
#endif

/****************************************************************/
// Perform basic initialization of the TouCAN hardware.
// The typical values for toucan are CAN_A, CAN_B, or CAN_C.

void TouCAN_init( struct TOUCAN_tag *toucan )
{
  // Described as CANMCR in the User's Manual, see Table 16-11.
  toucan->TCNMCR.B.SOFTRST = 1;  // Force a soft reset.
  asm("nop" ); // Delay a cycle before accessing MCR again.

  // wait for reset cycle to complete
  while ( toucan->TCNMCR.B.SOFTRST );  // busywait

  toucan->CANCTRL0.B.BOFFMSK = 0;  // bus off interrupt disabled
  toucan->CANCTRL0.B.ERRMSK = 0;   // error interrupt disabled

  // When connected to a Philips 82C251 transceiver, transmitting a
  // logic 0 is the "dominant" bus state, which is also received as a
  // logic zero.  Logic 1 is "recessive."
  toucan->CANCTRL0.B.RXMODE = 0;  // See Table 16-14
  toucan->CANCTRL0.B.TXMODE = 0;  // See Table 16-15
  
  // See Table 16-16 for the following.
  toucan->CANCTRL1.B.SAMP = 1;   // Take three samples of each received bit.
  toucan->CANCTRL1.B.TSYNC = 0;  // Timer synchronization disabled (can be used for synchronizing across network).
  toucan->CANCTRL1.B.LBUF = 1;   // Lowest numbered buffer is transmitted first (instead of lowest ID).

  // Set default timing parameters.
  TouCAN_set_rate( toucan, CAN_BUS_1MHZ );

  // Set all message buffers as inactive.
  {
    unsigned short buf;
    for ( buf = 0; buf<16 ; buf++ ) toucan->MBUFF[ buf ].SCR.B.CODE = 0;      // Mark buffer as inactive.
  }

  // Initialize the Receive Mask Registers.
  toucan->RXGMSKHI.R = 0;   // Set all receive masks to accept any standard format frame.
  toucan->RXGMSKLO.R = 0;
  toucan->RX14MSKHI.R = 0;
  toucan->RX14MSKLO.R = 0;
  toucan->RX15MSKHI.R = 0;
  toucan->RX15MSKLO.R = 0;

  // It is probably unnecessary to configure IMASK and IFLAG, they are
  // initialized by the soft reset.

  // IMASK is the Interrupt Mask register, see 16.7.13.
  toucan->IMASK.R = 0;  // Disable all per-buffer interrupts.

  // IFLAG is the Interrupt Flag registers, see 16.7.14.  To clear any
  // outstanding interrupt flags, they must be first be read (they
  // would be one), and then written as a zero.  The hardware
  // guarantees that new interrupts received between the read and the
  // write of any flag will prevent the write from changing the flag.
  {
    unsigned short tmp = toucan->IFLAG.R;
    toucan->IFLAG.R = 0;
  }
}
// Set the bus rate
void TouCAN_set_rate( struct TOUCAN_tag *toucan, TouCAN_rate_t rate )
{
  // Set all the timing parameters.
  // PRESDIV is the Prescaler Divide Register, see Table 16-8, and section 16.7.6.
  // See Table 16-18 for the CANCTRL2 timing settings.

  switch (rate) {

  default:
  case CAN_BUS_1MHZ:
    toucan->PRESDIV.R         = 1; // for 40mHz system clock, 1 MHz CAN bus rate: 20MHZ S-clock rate, 20 quanta per bit.
    toucan->CANCTRL1.B.PROPSE = 7; // Propagation segment time.
    toucan->CANCTRL2.B.PSEG   = 7; // Phase buffer segment 1.
    toucan->CANCTRL2.B.PSEG2  = 2; // Phase buffer segment 2.
    toucan->CANCTRL2.B.RJW    = 0; // Resynchronization jump width; must be less than PSEG.

    // BIT TIMING =
    //                           FSYS (40MHz)                                     40
    //   --------------------------------------------------------------- = ---------------- = 1MHz
    //   (1 + PRESDIV) * (1 + (PROPSEG + 1) + (PSEG1 + 1) + (PSEG2 + 1))     2 * (1+8+8+3)
    //
    break;

  case CAN_BUS_500KHZ:
    // 500 kHz bus
    toucan->PRESDIV.R         = 4; // Prescaler, 40 MHz system clock, 8MHz S-clock rate, 16 quanta per bit.
    toucan->CANCTRL1.B.PROPSE = 4; // Propagation segment time.
    toucan->CANCTRL2.B.PSEG   = 7; // Phase buffer segment 1.
    toucan->CANCTRL2.B.PSEG2  = 1; // Phase buffer segment 2.
    toucan->CANCTRL2.B.RJW    = 0; // Resynchronization jump width; must be less than PSEG.
    // 40 / (5 * (1 + 5 + 8 + 2)) = 40 / (5 * 16) = 0.5 MHz

    // With RJW==3, and the Toucan A sending to Toucan B, I observed a
    // 333kHZ bit rate instead of a 500kHz, both on the free-running
    // timer and on an oscilloscope looking at the actual bus.  The
    // actual resynchronization jump width is one more than the RJW
    // field, so even a value of 0 allows some clock rate skew.
  }
}
/****************************************************************/
// Clear the IFLAG bit for a buffer.  This works due to the special
// properties of IFLAG; bits can only be written to zero, and only if
// previously read as a one, with no intervening posting event.
//
// Note that contrary to the general Motorola big-endian philosophy,
// buffer 0 is assigned the LSB, i.e., bit 15 of the register in
// Motorola terminology.

static void clear_buffer_IFLAG( struct TOUCAN_tag *toucan, unsigned short buffer )
{
  unsigned short iflag_value = toucan->IFLAG.R;   // guarantee IFLAG has been read
  toucan->IFLAG.R = ~(1 << buffer);  // write back a mask with a single zero for this buffer
}

/****************************************************************/
// Initialize a buffer as a receive buffer. The buffer value is an
// index from 0 to 15.  For standard mode frames, the receive
// identifier is 11 bits; the specific bits that have significance are
// determined by the corresponding Receive Mask register.

void TouCAN_activate_receive_buffer( struct TOUCAN_tag *toucan, 
				     unsigned short buffer, 
				     unsigned short identifier )
{
  // De-activate the buffer.
  toucan->MBUFF[ buffer ].SCR.B.CODE = 0; 

  // Configure the buffer to receive a standard mode frame.
  // Set the ID[28-18] bits to the 11 bits of the identifier.
  // Set the RTR bit low.
  // Set the bottom four bits to zero to specify standard mode.
  toucan->MBUFF[ buffer ].ID_HIGH.R = (identifier & 0x7ff) << 5;

  // Clear the receive timestamp.
  toucan->MBUFF[ buffer ].ID_LOW.R = 0;        

  // Clear the data area.
  bzero( &toucan->MBUFF[ buffer ].DATA, 8 );

  // Clear any outstanding flag.
  clear_buffer_IFLAG( toucan, buffer );

  // Mark buffer for receive, active and empty.
  toucan->MBUFF[ buffer ].SCR.B.CODE = 4;      
}

/****************************************************************/
// Turn the controller on.  This is a separate operation so that the
// receive buffers may be configured after initialization, but before
// operation begins.

void TouCAN_turn_on( struct TOUCAN_tag *toucan )
{
  // Leave the HALT mode, the module will attempt to synchronize with the CAN bus.
  toucan->TCNMCR.B.HALT = 0;

  // Clear the FRZ bit so that the TouCAN will still run with the
  // FREEZE line is asserted; this means it will still operate even if
  // the BDM debugger halts the CPU.
  toucan->TCNMCR.B.FRZ = 0;
}

/****************************************************************/
// Transmit a message, using the specified controller, buffer, and
// message identifier.  The maximum data length is 8 bytes.
void TouCAN_send_message( struct TOUCAN_tag *toucan, 
			  unsigned short buffer, 
			  unsigned short identifier,
			  unsigned short length, void *data)
{
  // silently truncate overly long messages
  if ( length > 8 ) length = 8;

  // De-activate the buffer.
  toucan->MBUFF[ buffer ].SCR.B.CODE = 8; 

  // Configure the buffer to send a standard mode frame.
  // Set the ID[28-18] bits to the 11 bits of the identifier.
  // Set the RTR bit low (data frame).
  // Set the bottom four bits to be zero (standard mode).
  toucan->MBUFF[ buffer ].ID_HIGH.R = (identifier & 0x7ff) << 5;

  // Clear the receive timestamp.
  toucan->MBUFF[ buffer ].ID_LOW.R = 0;        

  // Fill the data area.
  memmove( &toucan->MBUFF[ buffer ].DATA, data, length );

  // Clear any outstanding flag.
  clear_buffer_IFLAG( toucan, buffer );

  // Set the message length and mark the buffer to transmit once, unconditionally.
  toucan->MBUFF[ buffer ].SCR.R = length | 0xc0;  // code = 12, also clears the time stamp
}

/****************************************************************/
// Read a message from a TouCAN buffer.  Three optional pointers are
// provided for return results:
//   length_ptr     to store the received message length
//   identifier_ptr to store the received message identifier
//   timestamp_ptr  to store the time stamp of the message
//   data           to store the received data, provided area must be at least 8 bytes long
//
// Upon a successful read, the buffer is reset to active reading.
// Note that the ID bits may have changed during reception if any were
// marked as "don't care" in the receive mask.  However, since they
// don't matter for buffer matching, they will be left in the new
// state.  However, this could cause problems if the receive masks are
// dynamically varied, which is outside the assumptions of this code.
//
// Returns an error indication: false if a message was received, true
// if the buffer was empty or inactive.

int TouCAN_read_receive_buffer( struct TOUCAN_tag *toucan,
				unsigned short buffer,
				unsigned short *length_ptr,
				unsigned short *identifier_ptr,
				unsigned short *timestamp_ptr,
				void *data )
{
  unsigned short length, timer, code;

  // Read the status value.  If not busy, this will also trigger the lock for this buffer.
  unsigned short status = toucan->MBUFF[ buffer ].SCR.R;
  code = status & 0xf0; // mask for CODE bits

  // If the buffer is a transmit buffer, inactive, or empty, release any lock and return an error.
  if ( code & 0x80 || code == 0x00 || code == 0x40 ) {
    // Read the free-running timer to release the lock on the buffer.
    timer = toucan->TIMER.R;
    return -1; 
  }
  
  // If the buffer is BUSY, it is currently being filled, busywait on
  // completion.  The hardware guarantees a maximum of 20 machine
  // cycles busy.
  if ( code & 0x10 ) {
    do {
      status = toucan->MBUFF[ buffer ].SCR.R;
    } while ( status & 0x10 );  // mask for BUSY
  }
    
  // At this point, the buffer is either FULL or OVERRUN.  Either way,
  // the data is valid, and any OVERRUN condition is ignored. Also,
  // the buffer is locked now.

  // Extract the message length.
  length = status & 0x0f;
  if (length > 8) length = 8;  // This should never happen.

  // Return the message metadata.
  if ( length_ptr != NULL ) *length_ptr = length;
  if ( identifier_ptr != NULL ) *identifier_ptr = (toucan->MBUFF[ buffer ].ID_HIGH.R >> 5) & 0x7ff;
  if ( timestamp_ptr != NULL) *timestamp_ptr = toucan->MBUFF[ buffer ].ID_LOW.R;

  // Copy the data to the user's buffer.
  if ( data != NULL) memmove( data,  &toucan->MBUFF[ buffer ].DATA, length );
  
  // Mark buffer for receive, active and empty.
  toucan->MBUFF[ buffer ].SCR.B.CODE = 4;      
  
  // Clear the receive indication.
  clear_buffer_IFLAG(toucan, buffer) ;

  // Read the free-running timer to release the lock on the buffer.
  timer = toucan->TIMER.R;

  return 0; // no error
}

// Read the interrupt flag for a buffer, which indicates successful
// transmission or reception.

unsigned short TouCAN_read_buffer_flag( struct TOUCAN_tag *toucan,
					unsigned short buffer )
{
  return ( toucan->IFLAG.R & (1 << buffer));
}
