// $Id: phyCORE_MPC565_flash.c,v 1.1 2005/03/15 22:24:31 garthz Exp $
//
// Simple interface to the external standard flash chips on the Phytec
// phyCORE-MPC565.  Derived from example code supplied on the Phytec
// CD-ROM.

// In general, this code writes a command sequence simultaneously to
// two 16-bit-wide AMD flash EEPROMs attached to the upper and lower
// halves of a 32 bit word.

#include "phyCORE_MPC565.h"
#include "phyCORE_MPC565_flash.h"

// The command codes are generally written to two addresses within the
// flash, 0x555 and 0x2aa.  However, the bottom two bits of the
// address bus don't go to the flash chips, since they select the byte
// within the 32-bit word presented by the two chips combined.
// Referring to these addresses via a unsigned int pointer effectively
// shifts them up two bits.

static volatile int *flash_base = (unsigned int *) PHY_STD_FLASH_BASE;

static unsigned short blink_count;  // for flashing the LED

/****************************************************************/
// Note the lack of error reporting.  I assume it is easier to run a
// separate verification than deal with propagating an error return
// value.

void phyCORE_MPC565_flash_write_word(unsigned int *address, unsigned int value)
{
  volatile unsigned int *flash_ptr = address;
  unsigned int status;
  unsigned short timeout = 0x1000;

  flash_base[ 0x555 ] = 0x00AA00AA;        /* program command sequence */
  flash_base[ 0x2aa ] = 0x00550055;
  flash_base[ 0x555 ] = 0x00A000A0;

  *flash_ptr = value; // program a word into both chips simultaneously
 
  do {
    status = *flash_ptr;   // read the programming status

    // Exit the loop when both chips are done, either by indicating
    // completion on DQ7, or by indicating timeout on DQ5.

    if ( ( ((status & 0x00800000) == (value & 0x00800000)) ||   // upper word complete
	   ((status & 0x00200000) != 0) ) &&                    // upper word timeout
	 ( ((status & 0x00000080) == (value & 0x00000080)) ||   // lower word complete
	   ((status & 0x00000020) != 0) ) )                     // lower word timeout
      break;

    // blink light while programming
    blink_count++;
    phyCORE_MPC565_set_LED( blink_count & 0x4000 );

  } while (timeout-- > 0);

  // If the write operation failed, that chip will need to be reset; this just resets both.
  if (( status & 0x00200020 ) != 0 || timeout == 0)
    phyCORE_MPC565_flash_reset();
  
}

// Erase the entire chip, which takes a while.
void phyCORE_MPC565_flash_erase_all(void)
{
  // Chip Erase sequence.
  flash_base[ 0x555 ] = 0x00AA00AA;
  flash_base[ 0x2aa ] = 0x00550055;
  flash_base[ 0x555 ] = 0x00800080;
  flash_base[ 0x555 ] = 0x00AA00AA;
  flash_base[ 0x2aa ] = 0x00550055;
  flash_base[ 0x555 ] = 0x00100010;    

  // Poll DQ7 on both chips until they go high to indicate a successful erase.
  // "During chip erase, a valid address is any non-protected sector address."
  // In principle, this could also examine DQ5 to monitor for failure.

  while((*flash_base & 0x00800080) != 0x00800080) 
    {
      // blink light while erasing
      blink_count++;
      phyCORE_MPC565_set_LED( blink_count & 0x8000);
    }

  phyCORE_MPC565_flash_reset();
}
// Erase a single sector, given an address within that sector.
extern void phyCORE_MPC565_flash_erase_sector(unsigned int *address)
{
  // Sector Erase sequence.
  flash_base[ 0x555 ] = 0x00AA00AA;
  flash_base[ 0x2aa ] = 0x00550055;
  flash_base[ 0x555 ] = 0x00800080;
  flash_base[ 0x555 ] = 0x00AA00AA;
  flash_base[ 0x2aa ] = 0x00550055;
  *address = 0x00300030;    

  // Poll DQ7 on both chips until they go high to indicate a successful erase.
  // "During chip erase, a valid address is any non-protected sector address."
  // In principle, this could also examine DQ5 to monitor for failure.

  while((*address & 0x00800080) != 0x00800080) 
    {
      // blink light while erasing
      blink_count++;
      phyCORE_MPC565_set_LED( blink_count & 0x8000);
    }

  phyCORE_MPC565_flash_reset();

}

void phyCORE_MPC565_flash_reset(void)
{
  // Reset sequence.  This returns the internal state machine to read access.
  flash_base[ 0x555 ] = 0x00AA00AA;
  flash_base[ 0x2aa ] = 0x00550055;
  flash_base[ 0x555 ] = 0x00F000F0;
}

unsigned int phyCORE_MPC565_flash_read_manuf_ID(void)
{
  unsigned int manuf_ID;
  // read Manufacturer ID command sequence
  flash_base[ 0x555 ] = 0x00AA00AA;
  flash_base[ 0x2aa ] = 0x00550055;
  flash_base[ 0x555 ] = 0x00900090;
  manuf_ID = flash_base[ 0 ];
  
  phyCORE_MPC565_flash_reset();

  return manuf_ID;
}    
unsigned int phyCORE_MPC565_flash_read_device_ID(void)
{
  unsigned int device_ID;
  // read Device ID command sequence
  flash_base[ 0x555 ] = 0x00AA00AA;
  flash_base[ 0x2aa ] = 0x00550055;
  flash_base[ 0x555 ] = 0x00900090;
  device_ID = flash_base[ 1 ];

  phyCORE_MPC565_flash_reset();

  return device_ID;
}    
